using System.Collections.Generic;
using System.Linq;
namespace Oxide.Plugins
{
    [Info("Realistic Weather", "OuTSMoKE, deano79, CaptainNebulous", "1.3.4", ResourceId = 2519)]
    [Description("Calls weather events with console commands.")]
    class RealisticWeather : RustPlugin
    {
        public Timer StormTimer;
        public int time = 0;
		public List<string> cmds = new List<string>
        {
            "rw rain",
            "rw lightrain",
            "rw longrain",
			"rw storm",
			"rw longstorm",
			"rw lightfog",
			"rw fog",
			"rw dust",
			"rw heavyfog",
			"rw overcast",
			"rw partlycloudy",
			"rw spottyshowers",
			"rw quickfront",
			"rw sunandclouds",
			"rw clear",
			"rw volcanicsky",
			"rw smokeysky",
			"rw windy"
        };

        void Unload()
        {
            if (StormTimer != null)
                StormTimer.Destroy();
            time = 0;
        }

        [ConsoleCommand("rw")]
        void cmdSpecial(ConsoleSystem.Arg arg)
        {
            if (!arg.IsServerside) return;
            if (arg.Args == null || arg.Args.Count() == 0) return;
            var stormType = arg.Args[0];
            if (stormType.Equals("random"))
            {
                stormType = Storms.Keys.ToList().GetRandom();
                arg.ReplyWith($"Running random weather event: {stormType}");
            }
            if (!Storms.ContainsKey(stormType)) return;
            NewStormTimer(stormType);
        }

        void NewStormTimer(string TypeOfStorm)
        {
            if (StormTimer != null)
                StormTimer.Destroy();
            time = 0;
            StormTimer = timer.Every(1, () => ProcessStorm(Storms[TypeOfStorm]));
        }

        void ProcessStorm(Dictionary<int, List<string>> StormData)
        {
            time++;
            if (StormData.ContainsKey(time))
                foreach (var entry in StormData[time])
                {
                    if (entry == "STOP") { StormTimer.Destroy(); time = 0; return; }
                    ConsoleSystem.Run(ConsoleSystem.Option.Server, entry);
                }
        }

        public Dictionary<string, Dictionary<int, List<string>>> Storms = new Dictionary<string, Dictionary<int, List<string>>>
                {
                    {"storm", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.cloud_coverage 0.02", "weather.cloud_sharpness 0.1", "weather.atmosphere_contrast 1.38" } },
						{10, new List<string> {"weather.cloud_coverage 0.04" } },
						{20, new List<string> {"weather.cloud_coverage 0.06", "weather.cloud_size 1.25" } },
						{30, new List<string> {"weather.cloud_coverage 0.08" } },
						{40, new List<string> {"weather.cloud_coverage 0.1", "weather.cloud_size 1.5" } },
						{50, new List<string> {"weather.cloud_coverage 0.12" } },
						{60, new List<string> {"weather.cloud_coverage 0.14", "weather.cloud_size 1.75" } },
						{70, new List<string> {"weather.cloud_coverage 0.16" } },
						{80, new List<string> {"weather.cloud_coverage 0.18" } },
						{90, new List<string> {"weather.cloud_coverage 0.2", "weather.cloud_size 2.0" } },
						{100, new List<string> {"weather.cloud_coverage 0.22", "weather.cloud_sharpness 0.2", "weather.atmosphere_contrast 1.75" } },
						{110, new List<string> {"weather.cloud_coverage 0.24" } },
						{120, new List<string> {"weather.cloud_coverage 0.26", "weather.cloud_size 2.25" } },
						{130, new List<string> {"weather.cloud_coverage 0.28" } },
						{140, new List<string> {"weather.cloud_coverage 0.3", "weather.cloud_size 2.5" } },
						{150, new List<string> {"weather.cloud_coverage 0.32" } },
						{160, new List<string> {"weather.cloud_coverage 0.34" } },
						{170, new List<string> {"weather.cloud_coverage 0.36", "weather.cloud_sharpness 0.3" } },
						{180, new List<string> {"weather.cloud_coverage 0.38" } },
						{190, new List<string> {"weather.cloud_coverage 0.4" } },
						{200, new List<string> {"weather.cloud_coverage 0.42" } },
						{210, new List<string> {"weather.cloud_coverage 0.44" } },
						{220, new List<string> {"weather.cloud_coverage 0.46" } },
						{230, new List<string> {"weather.cloud_coverage 0.48", "weather.cloud_sharpness 0.4" } },
						{240, new List<string> {"weather.cloud_coverage 0.5" } },
						{245, new List<string> {"weather.rain 0.01" } },
						{250, new List<string> {"weather.fog 0.02" } },
						{260, new List<string> {"weather.cloud_coverage 0.52", "weather.cloud_sharpness 0.5" } },
						{270, new List<string> {"weather.fog 0.04" } },
						{280, new List<string> {"weather.cloud_coverage 0.54" } },
						{290, new List<string> {"weather.fog 0.06" } },
						{300, new List<string> {"weather.cloud_coverage 0.56" } },
						{305, new List<string> {"weather.rain 0.02" } },
						{310, new List<string> {"weather.fog 0.08" } },
						{320, new List<string> {"weather.cloud_coverage 0.58" } },
						{330, new List<string> {"weather.fog 0.1" } },
						{340, new List<string> {"weather.cloud_coverage 0.6" } },
						{350, new List<string> {"weather.fog 0.12" } },
						{360, new List<string> {"weather.cloud_coverage 0.62" } },
						{365, new List<string> {"weather.rain 0.05" } },
						{370, new List<string> {"weather.fog 0.14" } },
						{380, new List<string> {"weather.cloud_coverage 0.64" } },
						{390, new List<string> {"weather.fog 0.16" } },
						{400, new List<string> {"weather.cloud_coverage 0.66", "weather.cloud_attenuation 0.1" } },
						{410, new List<string> {"weather.fog 0.18" } },
						{420, new List<string> {"weather.cloud_coverage 0.68" } },
						{425, new List<string> {"weather.rain 0.1" } },
						{430, new List<string> {"weather.fog 0.20" } },
						{440, new List<string> {"weather.cloud_coverage 0.7", "weather.cloud_attenuation 0.2" } },
						{450, new List<string> {"weather.fog 0.22" } },
						{460, new List<string> {"weather.cloud_coverage 0.72" } },
						{470, new List<string> {"weather.fog 0.24" } },
						{480, new List<string> {"weather.cloud_coverage 0.74", "weather.cloud_attenuation 0.3" } },
						{485, new List<string> {"weather.rain 0.2" } },
						{490, new List<string> {"weather.fog 0.26" } },
						{500, new List<string> {"weather.cloud_coverage 0.76", "weather.cloud_attenuation 0.4" } },
						{520, new List<string> {"weather.cloud_coverage 0.78" } },
						{540, new List<string> {"weather.cloud_coverage 0.8", "weather.cloud_attenuation 0.5" } },
						{545, new List<string> {"weather.rain 0.3" } },
						{560, new List<string> {"weather.cloud_coverage 0.82", "weather.cloud_attenuation 0.6" } },
						{580, new List<string> {"weather.cloud_coverage 0.84", "weather.cloud_attenuation 0.7" } },
						{600, new List<string> {"weather.cloud_coverage 0.86", "weather.cloud_attenuation 0.8" } },
						{605, new List<string> {"weather.rain 0.4", "weather.wind 0.2", "weather.thunder 1" } },
						{620, new List<string> {"weather.cloud_coverage 0.88" } },
						{640, new List<string> {"weather.cloud_coverage 0.9" } },
						{660, new List<string> {"weather.cloud_coverage 0.92" } },
						{665, new List<string> {"weather.rain 0.5" } },
						{680, new List<string> {"weather.cloud_coverage 0.94" } },
						{700, new List<string> {"weather.cloud_coverage 0.96" } },
						{720, new List<string> {"weather.cloud_coverage 0.98" } },
						{725, new List<string> {"weather.rain 0.6", "weather.wind 0.4" } },
						{740, new List<string> {"weather.cloud_coverage 1" } },
						{785, new List<string> {"weather.rain 0.7" } },
						{845, new List<string> {"weather.rain 0.8" } },
						{850, new List<string> {"weather.wind 0.6" } },
						{905, new List<string> {"weather.rain 0.9", "weather.wind 0.8" } },
						{965, new List<string> {"weather.rain 1", "weather.wind 1", "weather.cloud_size 2.0" } },
						{1035, new List<string> {"weather.rain 0.8" } },
						{1065, new List<string> {"weather.rain 0.7", "weather.wind 0.8" } },
						{1095, new List<string> {"weather.rain 0.6", "weather.wind 0.6", "weather.cloud_attenuation 0.7" } },
						{1160, new List<string> {"weather.wind 0.4", "weather.cloud_size 1.75" } },
						{1185, new List<string> {"weather.rain 0.4", "weather.rainbow 1" } },
						{1240, new List<string> {"weather.wind 0.2", "weather.cloud_attenuation 0.6" } },
						{1245, new List<string> {"weather.rain 0.3", "weather.cloud_size 1.5" } },
						{1250, new List<string> {"weather.cloud_coverage 0.98" } },
						{1270, new List<string> {"weather.cloud_coverage 0.96" } },
						{1285, new List<string> {"weather.rain 0.2", "weather.cloud_size 1.25" } },
						{1290, new List<string> {"weather.cloud_coverage 0.94" } },
						{1310, new List<string> {"weather.cloud_coverage 0.92", "weather.cloud_attenuation 0.5" } },
						{1330, new List<string> {"weather.cloud_coverage 0.9", "weather.cloud_size 1.0" } },
						{1335, new List<string> {"weather.rain 0.15", "weather.wind 0", "weather.thunder 0" } },
						{1340, new List<string> {"weather.cloud_attenuation 0.4" } },
						{1350, new List<string> {"weather.cloud_coverage 0.88" } },
						{1370, new List<string> {"weather.cloud_coverage 0.86" } },
						{1375, new List<string> {"weather.rain 0.1", "weather.cloud_size 0.88" } },
						{1390, new List<string> {"weather.cloud_coverage 0.84" } },
						{1405, new List<string> {"weather.rain 0.06" } },
						{1410, new List<string> {"weather.cloud_coverage 0.82" } },
						{1430, new List<string> {"weather.cloud_coverage 0.8", "weather.cloud_attenuation 0.3", "weather.cloud_size 0.75" } },
						{1450, new List<string> {"weather.cloud_coverage 0.78" } },
						{1455, new List<string> {"weather.rain 0.05" } },
						{1470, new List<string> {"weather.cloud_coverage 0.76" } },
						{1490, new List<string> {"weather.cloud_coverage 0.74" } },
						{1495, new List<string> {"weather.rain 0.04" } },
						{1500, new List<string> {"weather.fog 0.24" } },
						{1510, new List<string> {"weather.cloud_coverage 0.72" } },
						{1520, new List<string> {"weather.fog 0.22" } },
						{1530, new List<string> {"weather.cloud_coverage 0.7" } },
						{1535, new List<string> {"weather.rain 0.03" } },
						{1540, new List<string> {"weather.fog 0.2" } },
						{1550, new List<string> {"weather.cloud_coverage 0.68" } },
						{1560, new List<string> {"weather.fog 0.18" } },
						{1570, new List<string> {"weather.cloud_coverage 0.66", "weather.cloud_attenuation 0.2" } },
						{1575, new List<string> {"weather.rain 0.02" } },
						{1580, new List<string> {"weather.fog 0.16", } },
						{1590, new List<string> {"weather.cloud_coverage 0.64" } },
						{1600, new List<string> {"weather.fog 0.14" } },
						{1610, new List<string> {"weather.cloud_coverage 0.62" } },
						{1615, new List<string> {"weather.rain 0.01" } },
						{1620, new List<string> {"weather.fog 0.12" } },
						{1630, new List<string> {"weather.cloud_coverage 0.6" } },
						{1640, new List<string> {"weather.fog 0.1" } },
						{1650, new List<string> {"weather.cloud_coverage 0.58" } },
						{1660, new List<string> {"weather.fog 0.08" } },
						{1670, new List<string> {"weather.cloud_coverage 0.56" } },
						{1680, new List<string> {"weather.fog 0.06" } },
						{1690, new List<string> {"weather.cloud_coverage 0.54" } },
						{1700, new List<string> {"weather.fog 0.04" } },
						{1710, new List<string> {"weather.cloud_coverage 0.52" } },
						{1720, new List<string> {"weather.fog 0.02" } },
						{1730, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_attenuation 0.1" } },
						{1735, new List<string> {"weather.rain 0" } },
						{1740, new List<string> {"weather.fog 0" } },
						{1750, new List<string> {"weather.cloud_coverage 0.48", "weather.cloud_sharpness 0.4" } },
						{1760, new List<string> {"weather.cloud_coverage 0.46" } },
						{1770, new List<string> {"weather.cloud_coverage 0.44" } },
						{1780, new List<string> {"weather.cloud_coverage 0.42" } },
						{1790, new List<string> {"weather.cloud_coverage 0.4" } },
						{1800, new List<string> {"weather.cloud_coverage 0.38" } },
						{1810, new List<string> {"weather.cloud_coverage 0.36", "weather.cloud_sharpness 0.3" } },
						{1820, new List<string> {"weather.cloud_coverage 0.34", "weather.cloud_size 0.88" } },
						{1830, new List<string> {"weather.cloud_coverage 0.32" } },
						{1840, new List<string> {"weather.cloud_coverage 0.3" } },
						{1850, new List<string> {"weather.cloud_coverage 0.28", "weather.cloud_sharpness 0.2" } },
						{1860, new List<string> {"weather.cloud_coverage 0.26" } },
						{1870, new List<string> {"weather.cloud_coverage 0.24" } },
						{1880, new List<string> {"weather.cloud_coverage 0.22" } },
						{1890, new List<string> {"weather.cloud_coverage 0.2", "weather.cloud_size 0.875" } },
						{1900, new List<string> {"weather.cloud_coverage 0.18", "weather.cloud_attenuation -1" } },
						{1910, new List<string> {"weather.cloud_coverage 0.16", "weather.cloud_sharpness 0.1", "weather.atmosphere_contrast 1.25" } },
						{1920, new List<string> {"weather.cloud_coverage 0.14" } },
						{1930, new List<string> {"weather.cloud_coverage 0.12" } },
						{1940, new List<string> {"weather.cloud_coverage 0.1", "weather.cloud_size -1" } },
						{1950, new List<string> {"weather.cloud_coverage 0.08", "weather.cloud_sharpness -1", "weather.atmosphere_contrast -1" } },
						{1960, new List<string> {"weather.cloud_coverage 0.06" } },
						{1970, new List<string> {"weather.cloud_coverage 0.04" } },
						{1980, new List<string> {"weather.cloud_coverage 0.02", "weather.rainbow -1" } },						
                        {1990, new List<string> {"weather.cloud_coverage 0", "weather.reset", "STOP" } },
                    }
                    },
                    {"longstorm", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.cloud_coverage 0.02" } },
						{10, new List<string> {"weather.cloud_coverage 0.04" } },
						{20, new List<string> {"weather.cloud_coverage 0.06", "weather.cloud_size 1.38" } },
						{30, new List<string> {"weather.cloud_coverage 0.08", "weather.cloud_sharpness 0.1", "weather.atmosphere_contrast 1.5" } },
						{40, new List<string> {"weather.cloud_coverage 0.1", "weather.cloud_size 1.75" } },
						{50, new List<string> {"weather.cloud_coverage 0.12" } },
						{60, new List<string> {"weather.cloud_coverage 0.14", "weather.cloud_size 2.0" } },
						{70, new List<string> {"weather.cloud_coverage 0.16" } },
						{80, new List<string> {"weather.cloud_coverage 0.18", "weather.cloud_sharpness 0.2", "weather.atmosphere_contrast 2.0" } },
						{90, new List<string> {"weather.cloud_coverage 0.2", "weather.cloud_size 2.38" } },
						{100, new List<string> {"weather.cloud_coverage 0.22" } },
						{110, new List<string> {"weather.cloud_coverage 0.24", "weather.cloud_size 2.75" } },
						{120, new List<string> {"weather.cloud_coverage 0.26" } },
						{130, new List<string> {"weather.cloud_coverage 0.28", "weather.cloud_sharpness 0.3" } },
						{140, new List<string> {"weather.cloud_coverage 0.3", "weather.cloud_attenuation 0.1", "weather.cloud_size 3.0" } },
						{150, new List<string> {"weather.cloud_coverage 0.32", "weather.cloud_sharpness 0.4" } },
						{160, new List<string> {"weather.cloud_coverage 0.34" } },
						{170, new List<string> {"weather.cloud_coverage 0.36", "weather.cloud_sharpness 0.5" } },
						{180, new List<string> {"weather.cloud_coverage 0.38" } },
						{190, new List<string> {"weather.cloud_coverage 0.4", "weather.atmosphere_rayleigh 1.5" } },
						{200, new List<string> {"weather.cloud_coverage 0.42" } },
						{210, new List<string> {"weather.cloud_coverage 0.44", "weather.cloud_sharpness 0.6" } },
						{220, new List<string> {"weather.cloud_coverage 0.46", "weather.cloud_attenuation 0.2" } },
						{230, new List<string> {"weather.cloud_coverage 0.48" } },
						{240, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_sharpness 0.7" } },
						{245, new List<string> {"weather.rain 0.01" } },
						{250, new List<string> {"weather.fog 0.02" } },
						{260, new List<string> {"weather.cloud_coverage 0.52" } },
						{270, new List<string> {"weather.fog 0.04" } },
						{280, new List<string> {"weather.cloud_coverage 0.54", "weather.cloud_attenuation 0.3" } },
						{290, new List<string> {"weather.fog 0.06" } },
						{300, new List<string> {"weather.cloud_coverage 0.56" } },
						{305, new List<string> {"weather.rain 0.02" } },
						{310, new List<string> {"weather.fog 0.08" } },
						{320, new List<string> {"weather.cloud_coverage 0.58" } },
						{330, new List<string> {"weather.fog 0.1" } },
						{340, new List<string> {"weather.cloud_coverage 0.6", "weather.cloud_attenuation 0.4" } },
						{350, new List<string> {"weather.fog 0.12" } },
						{360, new List<string> {"weather.cloud_coverage 0.62" } },
						{365, new List<string> {"weather.rain 0.05" } },
						{370, new List<string> {"weather.fog 0.14" } },
						{380, new List<string> {"weather.cloud_coverage 0.64", "weather.cloud_attenuation 0.5" } },
						{390, new List<string> {"weather.fog 0.16" } },
						{400, new List<string> {"weather.cloud_coverage 0.66", "weather.cloud_attenuation 0.6" } },
						{410, new List<string> {"weather.fog 0.18" } },
						{420, new List<string> {"weather.cloud_coverage 0.68", "weather.thunder 1" } },
						{425, new List<string> {"weather.rain 0.1", "weather.cloud_attenuation 0.7" } },
						{430, new List<string> {"weather.fog 0.20" } },
						{440, new List<string> {"weather.cloud_coverage 0.7" } },
						{450, new List<string> {"weather.fog 0.22" } },
						{460, new List<string> {"weather.cloud_coverage 0.72", "weather.cloud_attenuation 0.8" } },
						{470, new List<string> {"weather.fog 0.24" } },
						{480, new List<string> {"weather.cloud_coverage 0.74" } },
						{485, new List<string> {"weather.rain 0.2" } },
						{490, new List<string> {"weather.fog 0.26" } },
						{500, new List<string> {"weather.cloud_coverage 0.76" } },
						{510, new List<string> {"weather.fog 0.28" } },
						{520, new List<string> {"weather.cloud_coverage 0.78", "weather.cloud_attenuation 0.9" } },
						{530, new List<string> {"weather.fog 0.3" } },
						{540, new List<string> {"weather.cloud_coverage 0.8", "weather.atmosphere_rayleigh -1" } },
						{545, new List<string> {"weather.rain 0.3" } },
						{550, new List<string> {"weather.fog 0.32" } },
						{560, new List<string> {"weather.cloud_coverage 0.82" } },
						{570, new List<string> {"weather.fog 0.34" } },
						{580, new List<string> {"weather.cloud_coverage 0.84" } },
						{590, new List<string> {"weather.fog 0.36" } },
						{600, new List<string> {"weather.cloud_coverage 0.86" } },
						{605, new List<string> {"weather.rain 0.4", "weather.wind 0.2" } },
						{610, new List<string> {"weather.fog 0.38" } },
						{620, new List<string> {"weather.cloud_coverage 0.88" } },
						{630, new List<string> {"weather.fog 0.4" } },
						{640, new List<string> {"weather.cloud_coverage 0.9", "weather.cloud_attenuation 1" } },
						{650, new List<string> {"weather.fog 0.42" } },
						{660, new List<string> {"weather.cloud_coverage 0.92" } },
						{665, new List<string> {"weather.rain 0.5" } },
						{670, new List<string> {"weather.fog 0.44" } },
						{680, new List<string> {"weather.cloud_coverage 0.94" } },
						{690, new List<string> {"weather.fog 0.46" } },
						{700, new List<string> {"weather.cloud_coverage 0.96" } },
						{710, new List<string> {"weather.fog 0.48" } },
						{720, new List<string> {"weather.cloud_coverage 0.98" } },
						{725, new List<string> {"weather.rain 0.6", "weather.wind 0.4" } },
						{730, new List<string> {"weather.fog 0.5" } },
						{740, new List<string> {"weather.cloud_coverage 1" } },
						{785, new List<string> {"weather.rain 0.7" } },
						{845, new List<string> {"weather.rain 0.8" } },
						{850, new List<string> {"weather.wind 0.6" } },
						{905, new List<string> {"weather.rain 0.9", "weather.wind 0.8" } },
						{965, new List<string> {"weather.rain 1", "weather.wind 1" } },
						{980, new List<string> {"weather.cloud_attenuation 0.9" } },
						{2095, new List<string> {"weather.rain 0.9", "weather.cloud_size 2.38" } },
						{2135, new List<string> {"weather.rain 0.8" } },
						{2165, new List<string> {"weather.rain 0.7", "weather.wind 0.8" } },
						{2170, new List<string> {"weather.cloud_attenuation 0.8", "weather.cloud_size 2.0" } },
						{2195, new List<string> {"weather.rain 0.6", "weather.wind 0.6" } },
						{2225, new List<string> {"weather.rain 0.5" } },
						{2260, new List<string> {"weather.wind 0.4", "weather.cloud_size 1.38" } },
						{2285, new List<string> {"weather.rain 0.4", "weather.cloud_attenuation 0.7" } },
						{2340, new List<string> {"weather.wind 0.2" } },
						{2345, new List<string> {"weather.rain 0.3" } },
						{2350, new List<string> {"weather.cloud_coverage 0.98", "weather.cloud_size 1.0" } },
						{2360, new List<string> {"weather.fog 0.48" } },
						{2370, new List<string> {"weather.cloud_coverage 0.96" } },
						{2380, new List<string> {"weather.fog 0.46" } },
						{2385, new List<string> {"weather.rain 0.2", "weather.cloud_size 0.88" } },
						{2390, new List<string> {"weather.cloud_coverage 0.94" } },
						{2400, new List<string> {"weather.fog 0.44" } },
						{2410, new List<string> {"weather.cloud_coverage 0.92" } },
						{2420, new List<string> {"weather.fog 0.42" } },
						{2430, new List<string> {"weather.cloud_coverage 0.9", "weather.cloud_size 0.75" } },
						{2435, new List<string> {"weather.rain 0.15", "weather.wind 0" } },
						{2440, new List<string> {"weather.fog 0.4", "weather.cloud_attenuation 0.6", "weather.rainbow 1" } },
						{2450, new List<string> {"weather.cloud_coverage 0.88" } },
						{2460, new List<string> {"weather.fog 0.38", "weather.cloud_size 0.63" } },
						{2470, new List<string> {"weather.cloud_coverage 0.86" } },
						{2475, new List<string> {"weather.rain 0.1", } },
						{2480, new List<string> {"weather.fog 0.36", "weather.cloud_size 0.5" } },
						{2490, new List<string> {"weather.cloud_coverage 0.84" } },
						{2500, new List<string> {"weather.fog 0.34" } },
						{2505, new List<string> {"weather.rain 0.06" } },
						{2510, new List<string> {"weather.cloud_coverage 0.82" } },
						{2520, new List<string> {"weather.fog 0.32" } },
						{2530, new List<string> {"weather.cloud_coverage 0.8", "weather.atmosphere_rayleigh 1.5", "weather.cloud_opacity 0.9" } },
						{2540, new List<string> {"weather.fog 0.3" } },
						{2550, new List<string> {"weather.cloud_coverage 0.78" } },
						{2555, new List<string> {"weather.rain 0.05" } },
						{2560, new List<string> {"weather.fog 0.28", "weather.cloud_size 0.63" } },
						{2570, new List<string> {"weather.cloud_coverage 0.76" } },
						{2580, new List<string> {"weather.fog 0.26" } },
						{2590, new List<string> {"weather.cloud_coverage 0.74" } },
						{2595, new List<string> {"weather.rain 0.04" } },
						{2600, new List<string> {"weather.fog 0.24", } },
						{2610, new List<string> {"weather.cloud_coverage 0.72" } },
						{2620, new List<string> {"weather.fog 0.22"} },
						{2630, new List<string> {"weather.cloud_coverage 0.7", "weather.cloud_attenuation 0.5", "weather.cloud_opacity 0.8" } },
						{2635, new List<string> {"weather.rain 0.03", "weather.cloud_size 0.75" } },
						{2640, new List<string> {"weather.fog 0.2" } },
						{2650, new List<string> {"weather.cloud_coverage 0.68" } },
						{2660, new List<string> {"weather.fog 0.18" } },
						{2670, new List<string> {"weather.cloud_coverage 0.66" } },
						{2675, new List<string> {"weather.rain 0.02", } },
						{2680, new List<string> {"weather.fog 0.16" } },
						{2690, new List<string> {"weather.cloud_coverage 0.64", "weather.cloud_attenuation 0.4" } },
						{2700, new List<string> {"weather.fog 0.14" } },
						{2710, new List<string> {"weather.cloud_coverage 0.62", "weather.thunder 0" } },
						{2715, new List<string> {"weather.rain 0.01" } },
						{2720, new List<string> {"weather.fog 0.12" } },
						{2730, new List<string> {"weather.cloud_coverage 0.6", "weather.cloud_opacity 0.7" } },
						{2740, new List<string> {"weather.fog 0.1", "weather.cloud_size 0.88" } },
						{2750, new List<string> {"weather.cloud_coverage 0.58" } },
						{2760, new List<string> {"weather.fog 0.08" } },
						{2770, new List<string> {"weather.cloud_coverage 0.56" } },
						{2780, new List<string> {"weather.fog 0.06" } },
						{2790, new List<string> {"weather.cloud_coverage 0.54" } },
						{2800, new List<string> {"weather.fog 0.04" } },
						{2810, new List<string> {"weather.cloud_coverage 0.52" } },
						{2820, new List<string> {"weather.fog 0.02" } },
						{2830, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_attenuation 0.3" } },
						{2835, new List<string> {"weather.rain 0" } },
						{3340, new List<string> {"weather.fog 0", "weather.cloud_size -1" } },
						{3350, new List<string> {"weather.cloud_coverage 0.48", "weather.cloud_sharpness 0.6" } },
						{3360, new List<string> {"weather.cloud_coverage 0.46", } },
						{3370, new List<string> {"weather.cloud_coverage 0.44" } },
						{3380, new List<string> {"weather.cloud_coverage 0.42" } },
						{3390, new List<string> {"weather.cloud_coverage 0.4", "weather.atmosphere_rayleigh -1" } },
						{3400, new List<string> {"weather.cloud_coverage 0.38", "weather.cloud_sharpness 0.5" } },
						{3410, new List<string> {"weather.cloud_coverage 0.36" } },
						{3420, new List<string> {"weather.cloud_coverage 0.34" } },
						{3430, new List<string> {"weather.cloud_coverage 0.32", "weather.cloud_attenuation 0.2" } },
						{3440, new List<string> {"weather.cloud_coverage 0.3", "weather.cloud_sharpness 0.4" } },
						{3450, new List<string> {"weather.cloud_coverage 0.28" } },
						{3460, new List<string> {"weather.cloud_coverage 0.26", "weather.cloud_opacity 0.8" } },
						{3470, new List<string> {"weather.cloud_coverage 0.24" } },
						{3480, new List<string> {"weather.cloud_coverage 0.22", "weather.cloud_sharpness 0.3" } },
						{3490, new List<string> {"weather.cloud_coverage 0.2" } },
						{3500, new List<string> {"weather.cloud_coverage 0.18" } },
						{3510, new List<string> {"weather.cloud_coverage 0.16", "weather.cloud_opacity 0.9" } },
						{3520, new List<string> {"weather.cloud_coverage 0.14", "weather.cloud_sharpness 0.2" } },
						{3530, new List<string> {"weather.cloud_coverage 0.12", "weather.cloud_attenuation 0.1" } },
						{3540, new List<string> {"weather.cloud_coverage 0.1" } },
						{3550, new List<string> {"weather.cloud_coverage 0.08", "weather.cloud_sharpness 0.1", "weather.atmosphere_contrast 1.5" } },
						{3560, new List<string> {"weather.cloud_coverage 0.06", "weather.cloud_attenuation -1" } },
						{3570, new List<string> {"weather.cloud_coverage 0.04", "weather.cloud_sharpness -1", "weather.atmosphere_contrast -1", "weather.cloud_opacity -1" } },
						{3580, new List<string> {"weather.cloud_coverage 0.02", "weather.rainbow -1" } },						
                        {3590, new List<string> {"weather.cloud_coverage 0", "weather.reset", "STOP" } },
                    }
                    },
					{"lightrain", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.cloud_coverage 0.01" } },
						{10, new List<string> {"weather.cloud_coverage 0.02" } },
						{20, new List<string> {"weather.cloud_coverage 0.04" } },
						{30, new List<string> {"weather.cloud_coverage 0.06" } },
						{40, new List<string> {"weather.cloud_coverage 0.08" } },
						{50, new List<string> {"weather.cloud_coverage 0.1" } },
						{60, new List<string> {"weather.cloud_coverage 0.12" } },
						{70, new List<string> {"weather.cloud_coverage 0.14" } },
						{80, new List<string> {"weather.cloud_coverage 0.16" } },
						{90, new List<string> {"weather.cloud_coverage 0.18" } },
						{100, new List<string> {"weather.cloud_coverage 0.2" } },
						{110, new List<string> {"weather.cloud_coverage 0.22" } },
						{120, new List<string> {"weather.cloud_coverage 0.24", "weather.cloud_size 1.25" } },
						{130, new List<string> {"weather.cloud_coverage 0.26" } },
						{140, new List<string> {"weather.cloud_coverage 0.28" } },
						{150, new List<string> {"weather.cloud_coverage 0.3" } },
						{160, new List<string> {"weather.cloud_coverage 0.32" } },
						{170, new List<string> {"weather.cloud_coverage 0.34" } },
						{180, new List<string> {"weather.cloud_coverage 0.36" } },
						{190, new List<string> {"weather.cloud_coverage 0.38" } },
						{200, new List<string> {"weather.cloud_coverage 0.4" } },
						{210, new List<string> {"weather.cloud_coverage 0.42" } },
						{220, new List<string> {"weather.cloud_coverage 0.44" } },
						{230, new List<string> {"weather.cloud_coverage 0.46" } },
						{240, new List<string> {"weather.cloud_coverage 0.48" } },
						{250, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_size 1.5" } },
						{260, new List<string> {"weather.cloud_coverage 0.52" } },
						{270, new List<string> {"weather.cloud_coverage 0.54" } },
						{280, new List<string> {"weather.cloud_coverage 0.56" } },
						{290, new List<string> {"weather.cloud_coverage 0.58" } },
						{300, new List<string> {"weather.cloud_coverage 0.6" } },
						{310, new List<string> {"weather.cloud_coverage 0.62" } },
						{320, new List<string> {"weather.cloud_coverage 0.64" } },
						{330, new List<string> {"weather.cloud_coverage 0.66" } },
						{340, new List<string> {"weather.cloud_coverage 0.68" } },
						{350, new List<string> {"weather.cloud_coverage 0.7" } },
						{360, new List<string> {"weather.cloud_coverage 0.72" } },
						{370, new List<string> {"weather.cloud_coverage 0.74" } },
						{380, new List<string> { "weather.cloud_coverage 0.75", "weather.cloud_attenuation 0.1" } },
						{390, new List<string> { "weather.fog 0.02" } },
						{400, new List<string> { "weather.cloud_coverage 0.76" } },
						{410, new List<string> { "weather.fog 0.04" } },
						{420, new List<string> { "weather.cloud_coverage 0.78" } },
						{430, new List<string> { "weather.fog 0.06" } },
						{440, new List<string> { "weather.cloud_coverage 0.8" } },
						{450, new List<string> { "weather.fog 0.08" } },
						{460, new List<string> { "weather.cloud_coverage 0.82" } },
						{470, new List<string> { "weather.fog 0.1" } },
                        {480, new List<string> { "weather.rain 0.01" } },
						{490, new List<string> { "weather.cloud_coverage 0.84" } },
						{500, new List<string> { "weather.fog 0.12" } },
                        {510, new List<string> { "weather.rain 0.02" } },
						{520, new List<string> { "weather.cloud_coverage 0.86" } },
						{540, new List<string> { "weather.cloud_coverage 0.88" } },
                        {560, new List<string> { "weather.rain 0.03" } },
						{570, new List<string> { "weather.cloud_coverage 0.9" } },
                        {590, new List<string> { "weather.rain 0.04" } },
						{600, new List<string> { "weather.cloud_coverage 0.92" } },
                        {620, new List<string> { "weather.rain 0.05" } },
						{630, new List<string> { "weather.cloud_coverage 0.94" } },
						{650, new List<string> { "weather.cloud_coverage 0.96" } },
						{670, new List<string> { "weather.cloud_coverage 0.98" } },
						{690, new List<string> { "weather.cloud_coverage 1", "weather.cloud_attenuation 0.2" } },
						{1040, new List<string> { "weather.cloud_coverage 0.98" } },
						{1050, new List<string> { "weather.cloud_coverage 0.96" } },
						{1060, new List<string> { "weather.cloud_coverage 0.94" } },
						{1070, new List<string> { "weather.cloud_coverage 0.92", "weather.cloud_attenuation 0.1" } },
						{1080, new List<string> { "weather.cloud_coverage 0.9" } },
                        {1090, new List<string> { "weather.rain 0.04" } },
						{1100, new List<string> { "weather.cloud_coverage 0.88" } },
						{1110, new List<string> { "weather.cloud_coverage 0.86" } },
						{1120, new List<string> { "weather.cloud_coverage 0.84" } },
						{1130, new List<string> { "weather.cloud_coverage 0.82" } },
						{1140, new List<string> { "weather.cloud_coverage 0.8" } },
                        {1150, new List<string> { "weather.rain 0.03" } },
						{1160, new List<string> { "weather.cloud_coverage 0.78" } },
						{1170, new List<string> { "weather.cloud_coverage 0.76" } },
						{1180, new List<string> { "weather.cloud_coverage 0.74", "weather.cloud_attenuation -1" } },
						{1190, new List<string> { "weather.cloud_coverage 0.72" } },
						{1200, new List<string> { "weather.cloud_coverage 0.7" } },
                        {1210, new List<string> { "weather.rain 0.02" } },
						{1220, new List<string> { "weather.cloud_coverage 0.68" } },
						{1230, new List<string> { "weather.cloud_coverage 0.66" } },
						{1240, new List<string> { "weather.cloud_coverage 0.64" } },
						{1250, new List<string> { "weather.cloud_coverage 0.62" } },
                        {1260, new List<string> { "weather.rain 0.01" } },
						{1270, new List<string> { "weather.cloud_coverage 0.6" } },
						{1280, new List<string> { "weather.cloud_coverage 0.58" } },
						{1290, new List<string> { "weather.cloud_coverage 0.56" } },
						{1300, new List<string> { "weather.cloud_coverage 0.54" } },
						{1310, new List<string> { "weather.cloud_coverage 0.52" } },
						{1320, new List<string> { "weather.cloud_coverage 0.5" } },
                        {1330, new List<string> { "weather.rain 0", "weather.cloud_size 1.25" } },
						{1350, new List<string> { "weather.cloud_coverage 0.48" } },
						{1370, new List<string> { "weather.cloud_coverage 0.46" } },
						{1390, new List<string> { "weather.cloud_coverage 0.44" } },
						{1410, new List<string> { "weather.cloud_coverage 0.42" } },
						{1430, new List<string> { "weather.cloud_coverage 0.4" } },
						{1450, new List<string> { "weather.cloud_coverage 0.38" } },
						{1470, new List<string> { "weather.cloud_coverage 0.36" } },
						{1490, new List<string> { "weather.cloud_coverage 0.34" } },
						{1510, new List<string> { "weather.cloud_coverage 0.32" } },
						{1530, new List<string> { "weather.cloud_coverage 0.3" } },
						{1550, new List<string> { "weather.cloud_coverage 0.28" } },
						{1570, new List<string> { "weather.cloud_coverage 0.26", "weather.cloud_size -1" } },
						{1580, new List<string> { "weather.fog 0.1" } },
						{1590, new List<string> { "weather.cloud_coverage 0.24" } },
						{1600, new List<string> { "weather.fog 0.08" } },
						{1610, new List<string> { "weather.cloud_coverage 0.22" } },
						{1620, new List<string> { "weather.fog 0.06" } },
						{1630, new List<string> { "weather.cloud_coverage 0.2" } },
						{1640, new List<string> { "weather.fog 0.04" } },
						{1650, new List<string> { "weather.cloud_coverage 0.18" } },
						{1660, new List<string> { "weather.fog 0.02" } },
						{1670, new List<string> { "weather.cloud_coverage 0.16" } },
						{1680, new List<string> { "weather.fog 0.01" } },
						{1690, new List<string> { "weather.cloud_coverage 0.14" } },
						{1700, new List<string> { "weather.fog 0" } },
						{1710, new List<string> { "weather.cloud_coverage 0.12" } },
						{1720, new List<string> { "weather.cloud_coverage 0.1" } },
						{1730, new List<string> { "weather.cloud_coverage 0.08" } },
						{1740, new List<string> { "weather.cloud_coverage 0.06" } },
						{1750, new List<string> { "weather.cloud_coverage 0.04" } },
						{1760, new List<string> { "weather.cloud_coverage 0.02" } },
                        {1770, new List<string> {"weather.cloud_coverage 0", "weather.reset", "STOP" } },
                    }
                    },
                    {"rain", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> { "weather.cloud_coverage 0.05" } },
                        {30, new List<string> { "weather.cloud_coverage 0.1", "weather.cloud_size 1.25" } },
                        {60, new List<string> { "weather.cloud_coverage 0.15" } },
                        {90, new List<string> { "weather.cloud_coverage 0.2", "weather.cloud_size 1.5" } },
                        {120, new List<string> { "weather.cloud_coverage 0.25" } },
                        {150, new List<string> { "weather.cloud_coverage 0.3" } },
                        {180, new List<string> { "weather.cloud_coverage 0.35" } },
                        {210, new List<string> { "weather.cloud_coverage 0.4" } },
                        {240, new List<string> { "weather.cloud_coverage 0.45" } },
                        {270, new List<string> { "weather.cloud_coverage 0.5" } },
                        {300, new List<string> { "weather.cloud_coverage 0.55" } },
                        {330, new List<string> { "weather.cloud_coverage 0.6", "weather.cloud_attenuation 0.1" } },
                        {340, new List<string> { "weather.rain 0.02" } },
						{360, new List<string> { "weather.cloud_coverage 0.85" } },
						{370, new List<string> { "weather.rain 0.2" } },
                        {390, new List<string> { "weather.fog 0.05" } },
                        {400, new List<string> { "weather.rain 0.3" } },
                        {410, new List<string> { "weather.cloud_coverage 0.9", "weather.cloud_attenuation 0.2" } },
                        {420, new List<string> { "weather.fog 0.1" } },
                        {430, new List<string> { "weather.rain 0.4" } },
                        {450, new List<string> { "weather.fog 0.15" } },
                        {440, new List<string> { "weather.cloud_coverage 0.95", "weather.cloud_sharpness 0.1", "weather.atmosphere_contrast 1.25" } },
                        {460, new List<string> { "weather.rain 0.5" } },
                        {470, new List<string> { "weather.cloud_coverage 1", "weather.cloud_attenuation 0.3" } },
                        {480, new List<string> { "weather.fog 0.2" } },
						{500, new List<string> { "weather.fog 0.25" } },
						{520, new List<string> { "weather.cloud_sharpness 0.2", "weather.atmosphere_contrast 1.5" } },
						{600, new List<string> { "weather.cloud_attenuation 0.4" } },
						{2200, new List<string> { "weather.rain 0.4" } },
						{2230, new List<string> { "weather.rain 0.3" } },
						{2260, new List<string> { "weather.rain 0.2" } },
						{2280, new List<string> { "weather.cloud_coverage 0.95" } },
						{2290, new List<string> { "weather.rain 0.1" } },
						{2295, new List<string> { "weather.cloud_coverage 0.9" } },
						{2310, new List<string> { "weather.cloud_coverage 0.85", "weather.cloud_attenuation 0.3" } },
						{2320, new List<string> { "weather.rain 0.05" } },
						{2340, new List<string> { "weather.cloud_coverage 0.80" } },
						{2350, new List<string> { "weather.rain 0.02" } },
						{2370, new List<string> { "weather.cloud_coverage 0.75" } },
						{2380, new List<string> { "weather.rain 0", "weather.cloud_size 1.25" } },
						{2400, new List<string> { "weather.cloud_coverage 0.7", "weather.cloud_attenuation 0.2" } },
						{2410, new List<string> { "weather.fog 0.2" } },
						{2420, new List<string> { "weather.cloud_coverage 0.65" } },
						{2430, new List<string> { "weather.fog 0.15" } },
						{2440, new List<string> { "weather.cloud_coverage 0.6", "weather.cloud_attenuation 0.1" } },
						{2450, new List<string> { "weather.fog 0.1" } },
						{2460, new List<string> { "weather.cloud_coverage 0.55" } },
						{2470, new List<string> { "weather.fog 0.05" } },
						{2480, new List<string> { "weather.cloud_coverage 0.5" } },
						{2500, new List<string> { "weather.fog 0" } },
						{2510, new List<string> { "weather.cloud_coverage 0.45", "weather.cloud_sharpness 0.1", "weather.atmosphere_contrast 1.25" } },
						{2530, new List<string> { "weather.cloud_coverage 0.4" } },
						{2550, new List<string> { "weather.cloud_coverage 0.35" } },
						{2570, new List<string> { "weather.cloud_coverage 0.3", "weather.cloud_attenuation -1" } },
						{2590, new List<string> { "weather.cloud_coverage 0.25" } },
						{2610, new List<string> { "weather.cloud_coverage 0.20", "weather.cloud_size -1" } },
						{2630, new List<string> { "weather.cloud_coverage 0.15", "weather.cloud_sharpness -1", "weather.atmosphere_contrast -1" } },
						{2650, new List<string> { "weather.cloud_coverage 0.1" } },
						{2670, new List<string> { "weather.cloud_coverage 0.05" } },
                        {2690, new List<string> {"weather.cloud_coverage 0", "weather.reset", "STOP" } },
                    }
                    },
                    {"longrain", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> { "weather.cloud_coverage 0.05" } },
                        {30, new List<string> { "weather.cloud_coverage 0.1", "weather.cloud_size 1.25" } },
                        {60, new List<string> { "weather.cloud_coverage 0.15" } },
                        {90, new List<string> { "weather.cloud_coverage 0.2", "weather.cloud_size 1.5" } },
                        {120, new List<string> { "weather.cloud_coverage 0.25" } },
                        {150, new List<string> { "weather.cloud_coverage 0.3", "weather.cloud_size 1.75" } },
                        {180, new List<string> { "weather.cloud_coverage 0.35", "weather.cloud_sharpness 0.1", "weather.atmosphere_contrast 1.38" } },
                        {210, new List<string> { "weather.cloud_coverage 0.4", "weather.cloud_size 2.0" } },
                        {240, new List<string> { "weather.cloud_coverage 0.45" } },
                        {270, new List<string> { "weather.cloud_coverage 0.5", "weather.cloud_attenuation 0.15" } },
                        {300, new List<string> { "weather.cloud_coverage 0.55" } },
                        {330, new List<string> { "weather.cloud_coverage 0.6" } },
                        {340, new List<string> { "weather.rain 0.02", "weather.cloud_sharpness 0.2", "weather.atmosphere_contrast 1.75" } },
						{360, new List<string> { "weather.cloud_coverage 0.85", "weather.cloud_attenuation 0.3" } },
						{370, new List<string> { "weather.rain 0.2" } },
                        {390, new List<string> { "weather.fog 0.05", "weather.cloud_attenuation 0.4" } },
                        {400, new List<string> { "weather.rain 0.3" } },
                        {410, new List<string> { "weather.cloud_coverage 0.9" } },
                        {420, new List<string> { "weather.fog 0.1" } },
                        {430, new List<string> { "weather.rain 0.4" } },
                        {450, new List<string> { "weather.fog 0.15" } },
                        {440, new List<string> { "weather.cloud_coverage 0.95", "weather.cloud_attenuation 0.5" } },
                        {460, new List<string> { "weather.rain 0.5" } },
                        {470, new List<string> { "weather.cloud_coverage 1" } },
                        {480, new List<string> { "weather.fog 0.2", "weather.cloud_sharpness 0.3" } },
						{500, new List<string> { "weather.fog 0.25" } },
						{520, new List<string> { "weather.fog 0.3" } },
						{540, new List<string> { "weather.fog 0.35" } },
						{560, new List<string> { "weather.fog 0.4" } },
						{580, new List<string> { "weather.fog 0.45" } },
						{600, new List<string> { "weather.fog 0.5" } },
						{2800, new List<string> { "weather.rain 0.4", "weather.cloud_attenuation 0.4" } },
						{2830, new List<string> { "weather.rain 0.3" } },
						{2860, new List<string> { "weather.rain 0.2" } },
						{2880, new List<string> { "weather.cloud_coverage 0.95" } },
						{2890, new List<string> { "weather.rain 0.1", "weather.cloud_attenuation 0.3" } },
						{2895, new List<string> { "weather.cloud_coverage 0.9" } },
						{2900, new List<string> { "weather.fog 0.45" } },
						{2910, new List<string> { "weather.cloud_coverage 0.85" } },
						{2920, new List<string> { "weather.rain 0.05", "weather.cloud_attenuation 0.2" } },
						{2930, new List<string> { "weather.fog 0.4", "weather.cloud_sharpness 0.2" } },
						{2940, new List<string> { "weather.cloud_coverage 0.80", "weather.atmosphere_rayleigh 1.5" } },
						{2950, new List<string> { "weather.rain 0.02" } },
						{2960, new List<string> { "weather.fog 0.35" } },
						{2970, new List<string> { "weather.cloud_coverage 0.75", "weather.cloud_attenuation 0.1" } },
						{2980, new List<string> { "weather.rain 0" } },
						{2990, new List<string> { "weather.fog 0.25", "weather.rainbow 1" } },
						{3000, new List<string> { "weather.cloud_coverage 0.7" } },
						{3010, new List<string> { "weather.fog 0.2", "weather.cloud_size 1.75" } },
						{3020, new List<string> { "weather.cloud_coverage 0.65" } },
						{3030, new List<string> { "weather.fog 0.15" } },
						{3040, new List<string> { "weather.cloud_coverage 0.6", "weather.cloud_attenuation -1" } },
						{3050, new List<string> { "weather.fog 0.1" } },
						{3060, new List<string> { "weather.cloud_coverage 0.55" } },
						{3070, new List<string> { "weather.fog 0.05" } },
						{3080, new List<string> { "weather.cloud_coverage 0.5" } },
						{3100, new List<string> { "weather.fog 0" } },
						{3110, new List<string> { "weather.cloud_coverage 0.45" } },
						{3130, new List<string> { "weather.cloud_coverage 0.4", "weather.atmosphere_rayleigh -1", "weather.cloud_size 1.5" } },
						{3150, new List<string> { "weather.cloud_coverage 0.35" } },
						{3170, new List<string> { "weather.cloud_coverage 0.3", "weather.cloud_sharpness 0.1", "weather.atmosphere_contrast 1.38" } },
						{3190, new List<string> { "weather.cloud_coverage 0.25", "weather.cloud_size 1.25" } },
						{3210, new List<string> { "weather.cloud_coverage 0.20", "weather.cloud_size -1" } },
						{3230, new List<string> { "weather.cloud_coverage 0.15" } },
						{3250, new List<string> { "weather.cloud_coverage 0.1", "weather.cloud_sharpness -1", "weather.atmosphere_contrast -1" } },
						{3270, new List<string> { "weather.cloud_coverage 0.05", "weather.rainbow -1" } },
                        {3290, new List<string> {"weather.cloud_coverage 0", "weather.reset", "STOP" } },
                    }
                    },
					{"lightfog", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.cloud_coverage 0.02" } },
						{10, new List<string> {"weather.cloud_coverage 0.04" } },
						{20, new List<string> {"weather.cloud_coverage 0.06" } },
						{30, new List<string> {"weather.cloud_coverage 0.08" } },
						{40, new List<string> {"weather.cloud_coverage 0.1" } },
						{50, new List<string> {"weather.cloud_coverage 0.12" } },
						{60, new List<string> {"weather.cloud_coverage 0.14" } },
						{70, new List<string> {"weather.cloud_coverage 0.16" } },
						{80, new List<string> {"weather.cloud_coverage 0.18" } },
						{90, new List<string> {"weather.cloud_coverage 0.2" } },
						{100, new List<string> {"weather.cloud_coverage 0.22" } },
						{110, new List<string> {"weather.cloud_coverage 0.24" } },
						{120, new List<string> {"weather.cloud_coverage 0.26" } },
						{130, new List<string> {"weather.cloud_coverage 0.28" } },
						{140, new List<string> {"weather.cloud_coverage 0.3" } },
						{150, new List<string> {"weather.cloud_coverage 0.32" } },
						{160, new List<string> {"weather.cloud_coverage 0.34" } },
						{170, new List<string> {"weather.cloud_coverage 0.36" } },
						{180, new List<string> {"weather.cloud_coverage 0.38" } },
						{190, new List<string> {"weather.cloud_coverage 0.4" } },
						{200, new List<string> {"weather.cloud_coverage 0.42" } },
						{210, new List<string> {"weather.cloud_coverage 0.44" } },
						{220, new List<string> {"weather.cloud_coverage 0.46" } },
						{230, new List<string> {"weather.cloud_coverage 0.48" } },
						{240, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_attenuation 0.1", "weather.atmosphere_contrast 0.9", "weather.cloud_coloring 0.8" } },
						{250, new List<string> {"weather.fog 0.02" } },
						{270, new List<string> {"weather.fog 0.04" } },
						{290, new List<string> {"weather.fog 0.06" } },
						{310, new List<string> {"weather.fog 0.08" } },
						{330, new List<string> {"weather.fog 0.1" } },
						{350, new List<string> {"weather.fog 0.12" } },
						{370, new List<string> {"weather.fog 0.14" } },
						{390, new List<string> {"weather.fog 0.16" } },
						{410, new List<string> {"weather.fog 0.18" } },
						{430, new List<string> {"weather.fog 0.20" } },
						{450, new List<string> {"weather.fog 0.22" } },
						{470, new List<string> {"weather.fog 0.24" } },
						{490, new List<string> {"weather.fog 0.25" } },
						{1200, new List<string> {"weather.fog 0.24" } },
						{1220, new List<string> {"weather.fog 0.22" } },
						{1240, new List<string> {"weather.fog 0.2" } },
						{1260, new List<string> {"weather.fog 0.18" } },
						{1280, new List<string> {"weather.fog 0.16" } },
						{1300, new List<string> {"weather.fog 0.14" } },
						{1320, new List<string> {"weather.fog 0.12" } },
						{1340, new List<string> {"weather.fog 0.1" } },
						{1360, new List<string> {"weather.fog 0.08" } },
						{1380, new List<string> {"weather.fog 0.06" } },
						{1400, new List<string> {"weather.fog 0.04" } },
						{1420, new List<string> {"weather.fog 0.02" } },
						{1430, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_attenuation -1", "weather.atmosphere_contrast -1", "weather.cloud_coloring -1" } },
						{1440, new List<string> {"weather.fog 0" } },
						{1450, new List<string> {"weather.cloud_coverage 0.48" } },
						{1460, new List<string> {"weather.cloud_coverage 0.46" } },
						{1470, new List<string> {"weather.cloud_coverage 0.44" } },
						{1480, new List<string> {"weather.cloud_coverage 0.42" } },
						{1490, new List<string> {"weather.cloud_coverage 0.4" } },
						{1500, new List<string> {"weather.cloud_coverage 0.38" } },
						{1510, new List<string> {"weather.cloud_coverage 0.36" } },
						{1520, new List<string> {"weather.cloud_coverage 0.34" } },
						{1530, new List<string> {"weather.cloud_coverage 0.32" } },
						{1540, new List<string> {"weather.cloud_coverage 0.3" } },
						{1550, new List<string> {"weather.cloud_coverage 0.28" } },
						{1560, new List<string> {"weather.cloud_coverage 0.26" } },
						{1570, new List<string> {"weather.cloud_coverage 0.24" } },
						{1580, new List<string> {"weather.cloud_coverage 0.22" } },
						{1590, new List<string> {"weather.cloud_coverage 0.2" } },
						{1600, new List<string> {"weather.cloud_coverage 0.18" } },
						{1610, new List<string> {"weather.cloud_coverage 0.16" } },
						{1620, new List<string> {"weather.cloud_coverage 0.14" } },
						{1630, new List<string> {"weather.cloud_coverage 0.12" } },
						{1640, new List<string> {"weather.cloud_coverage 0.1" } },
						{1650, new List<string> {"weather.cloud_coverage 0.08" } },
						{1660, new List<string> {"weather.cloud_coverage 0.06" } },
						{1670, new List<string> {"weather.cloud_coverage 0.04" } },
						{1680, new List<string> {"weather.cloud_coverage 0.02" } },						
                        {1690, new List<string> {"weather.cloud_coverage 0", "weather.reset", "STOP" } },
                    }
                    },
					{"fog", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.cloud_coverage 0.02" } },
						{10, new List<string> {"weather.cloud_coverage 0.04" } },
						{20, new List<string> {"weather.cloud_coverage 0.06" } },
						{30, new List<string> {"weather.cloud_coverage 0.08" } },
						{40, new List<string> {"weather.cloud_coverage 0.1" } },
						{50, new List<string> {"weather.cloud_coverage 0.12" } },
						{60, new List<string> {"weather.cloud_coverage 0.14" } },
						{70, new List<string> {"weather.cloud_coverage 0.16" } },
						{80, new List<string> {"weather.cloud_coverage 0.18" } },
						{90, new List<string> {"weather.cloud_coverage 0.2" } },
						{100, new List<string> {"weather.cloud_coverage 0.22" } },
						{110, new List<string> {"weather.cloud_coverage 0.24" } },
						{120, new List<string> {"weather.cloud_coverage 0.26" } },
						{130, new List<string> {"weather.cloud_coverage 0.28" } },
						{140, new List<string> {"weather.cloud_coverage 0.3", "weather.cloud_attenuation 0.1", "weather.atmosphere_contrast 0.9", "weather.cloud_coloring 0.8" } },
						{150, new List<string> {"weather.cloud_coverage 0.32" } },
						{160, new List<string> {"weather.cloud_coverage 0.34" } },
						{170, new List<string> {"weather.cloud_coverage 0.36" } },
						{180, new List<string> {"weather.cloud_coverage 0.38" } },
						{190, new List<string> {"weather.cloud_coverage 0.4" } },
						{200, new List<string> {"weather.cloud_coverage 0.42" } },
						{210, new List<string> {"weather.cloud_coverage 0.44" } },
						{220, new List<string> {"weather.cloud_coverage 0.46" } },
						{230, new List<string> {"weather.cloud_coverage 0.48" } },
						{240, new List<string> {"weather.cloud_coverage 0.5" } },
						{250, new List<string> {"weather.fog 0.02" } },
						{260, new List<string> {"weather.cloud_coverage 0.52" } },
						{270, new List<string> {"weather.fog 0.04" } },
						{280, new List<string> {"weather.cloud_coverage 0.54" } },
						{290, new List<string> {"weather.fog 0.06" } },
						{300, new List<string> {"weather.cloud_coverage 0.56" } },
						{310, new List<string> {"weather.fog 0.08" } },
						{320, new List<string> {"weather.cloud_coverage 0.58" } },
						{330, new List<string> {"weather.fog 0.1" } },
						{340, new List<string> {"weather.cloud_coverage 0.6", "weather.cloud_attenuation 0.2", "weather.atmosphere_contrast 0.8", "weather.cloud_coloring 0.6" } },
						{350, new List<string> {"weather.fog 0.12" } },
						{360, new List<string> {"weather.cloud_coverage 0.62" } },
						{370, new List<string> {"weather.fog 0.14" } },
						{380, new List<string> {"weather.cloud_coverage 0.64" } },
						{390, new List<string> {"weather.fog 0.16" } },
						{400, new List<string> {"weather.cloud_coverage 0.66" } },
						{410, new List<string> {"weather.fog 0.18" } },
						{420, new List<string> {"weather.cloud_coverage 0.68" } },
						{430, new List<string> {"weather.fog 0.20" } },
						{440, new List<string> {"weather.cloud_coverage 0.7" } },
						{450, new List<string> {"weather.fog 0.22" } },
						{460, new List<string> {"weather.cloud_coverage 0.72" } },
						{470, new List<string> {"weather.fog 0.24" } },
						{480, new List<string> {"weather.cloud_coverage 0.74" } },
						{490, new List<string> {"weather.fog 0.26" } },
						{510, new List<string> {"weather.fog 0.28" } },
						{530, new List<string> {"weather.fog 0.3" } },
						{550, new List<string> {"weather.fog 0.32" } },
						{570, new List<string> {"weather.fog 0.34" } },
						{590, new List<string> {"weather.fog 0.36" } },
						{610, new List<string> {"weather.fog 0.38" } },
						{630, new List<string> {"weather.fog 0.4" } },
						{2440, new List<string> {"weather.fog 0.4" } },
						{2460, new List<string> {"weather.fog 0.38" } },
						{2480, new List<string> {"weather.fog 0.36" } },
						{2500, new List<string> {"weather.fog 0.34" } },
						{2520, new List<string> {"weather.fog 0.32" } },
						{2540, new List<string> {"weather.fog 0.3" } },
						{2560, new List<string> {"weather.fog 0.28" } },
						{2580, new List<string> {"weather.fog 0.26" } },
						{2590, new List<string> {"weather.cloud_coverage 0.74" } },
						{2600, new List<string> {"weather.fog 0.24" } },
						{2610, new List<string> {"weather.cloud_coverage 0.72" } },
						{2620, new List<string> {"weather.fog 0.22" } },
						{2630, new List<string> {"weather.cloud_coverage 0.7" } },
						{2640, new List<string> {"weather.fog 0.2" } },
						{2650, new List<string> {"weather.cloud_coverage 0.68" } },
						{2660, new List<string> {"weather.fog 0.18" } },
						{2670, new List<string> {"weather.cloud_coverage 0.66" } },
						{2680, new List<string> {"weather.fog 0.16" } },
						{2690, new List<string> {"weather.cloud_coverage 0.64" } },
						{2700, new List<string> {"weather.fog 0.14" } },
						{2710, new List<string> {"weather.cloud_coverage 0.62" } },
						{2720, new List<string> {"weather.fog 0.12" } },
						{2730, new List<string> {"weather.cloud_coverage 0.6", "weather.cloud_attenuation 0.1", "weather.atmosphere_contrast 0.9", "weather.cloud_coloring 0.8" } },
						{2740, new List<string> {"weather.fog 0.1" } },
						{2750, new List<string> {"weather.cloud_coverage 0.58" } },
						{2760, new List<string> {"weather.fog 0.08" } },
						{2770, new List<string> {"weather.cloud_coverage 0.56" } },
						{2780, new List<string> {"weather.fog 0.06" } },
						{2790, new List<string> {"weather.cloud_coverage 0.54" } },
						{2800, new List<string> {"weather.fog 0.04" } },
						{2810, new List<string> {"weather.cloud_coverage 0.52" } },
						{2820, new List<string> {"weather.fog 0.02" } },
						{2830, new List<string> {"weather.cloud_coverage 0.5" } },
						{2840, new List<string> {"weather.fog 0" } },
						{2850, new List<string> {"weather.cloud_coverage 0.48" } },
						{2860, new List<string> {"weather.cloud_coverage 0.46" } },
						{2870, new List<string> {"weather.cloud_coverage 0.44" } },
						{2880, new List<string> {"weather.cloud_coverage 0.42" } },
						{2890, new List<string> {"weather.cloud_coverage 0.4" } },
						{2900, new List<string> {"weather.cloud_coverage 0.38" } },
						{2910, new List<string> {"weather.cloud_coverage 0.36" } },
						{2920, new List<string> {"weather.cloud_coverage 0.34" } },
						{2930, new List<string> {"weather.cloud_coverage 0.32" } },
						{2940, new List<string> {"weather.cloud_coverage 0.3", "weather.cloud_attenuation -1", "weather.atmosphere_contrast -1", "weather.cloud_coloring -1" } },
						{2950, new List<string> {"weather.cloud_coverage 0.28" } },
						{2960, new List<string> {"weather.cloud_coverage 0.26" } },
						{2970, new List<string> {"weather.cloud_coverage 0.24" } },
						{2980, new List<string> {"weather.cloud_coverage 0.22" } },
						{2990, new List<string> {"weather.cloud_coverage 0.2" } },
						{3000, new List<string> {"weather.cloud_coverage 0.18" } },
						{3010, new List<string> {"weather.cloud_coverage 0.16" } },
						{3020, new List<string> {"weather.cloud_coverage 0.14" } },
						{3030, new List<string> {"weather.cloud_coverage 0.12" } },
						{3040, new List<string> {"weather.cloud_coverage 0.1" } },
						{3050, new List<string> {"weather.cloud_coverage 0.08" } },
						{3060, new List<string> {"weather.cloud_coverage 0.06" } },
						{3070, new List<string> {"weather.cloud_coverage 0.04" } },
						{3080, new List<string> {"weather.cloud_coverage 0.02" } },						
                        {3090, new List<string> {"weather.cloud_coverage 0", "weather.reset", "STOP" } },
                    }
                    },
					{"dust", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.cloud_coverage 0.02" } },
						{10, new List<string> {"weather.cloud_coverage 0.04" } },
						{20, new List<string> {"weather.cloud_coverage 0.06" } },
						{30, new List<string> {"weather.cloud_coverage 0.08" } },
						{40, new List<string> {"weather.cloud_coverage 0.1", "weather.cloud_size 0.9" } },
						{50, new List<string> {"weather.cloud_coverage 0.12" } },
						{60, new List<string> {"weather.cloud_coverage 0.14" } },
						{70, new List<string> {"weather.cloud_coverage 0.16" } },
						{80, new List<string> {"weather.cloud_coverage 0.18" } },
						{90, new List<string> {"weather.cloud_coverage 0.2", "weather.cloud_size 0.8" } },
						{100, new List<string> {"weather.cloud_coverage 0.22" } },
						{110, new List<string> {"weather.cloud_coverage 0.24" } },
						{120, new List<string> {"weather.cloud_coverage 0.26" } },
						{130, new List<string> {"weather.cloud_coverage 0.28" } },
						{140, new List<string> {"weather.cloud_coverage 0.3", "weather.atmosphere_rayleigh 1.5", "weather.atmosphere_contrast 1.5", "weather.cloud_size 0.7" } },
						{150, new List<string> {"weather.cloud_coverage 0.32" } },
						{160, new List<string> {"weather.cloud_coverage 0.34" } },
						{170, new List<string> {"weather.cloud_coverage 0.36" } },
						{180, new List<string> {"weather.cloud_coverage 0.38" } },
						{190, new List<string> {"weather.cloud_coverage 0.4" } },
						{200, new List<string> {"weather.cloud_coverage 0.42" } },
						{210, new List<string> {"weather.cloud_coverage 0.44" } },
						{220, new List<string> {"weather.cloud_coverage 0.46" } },
						{230, new List<string> {"weather.cloud_coverage 0.48" } },
						{240, new List<string> {"weather.cloud_coverage 0.5", "weather.atmosphere_rayleigh 2.0", "weather.atmosphere_contrast 2.0" } },
						{250, new List<string> {"weather.fog 0.02" } },
						{270, new List<string> {"weather.fog 0.04" } },
						{290, new List<string> {"weather.fog 0.06" } },
						{310, new List<string> {"weather.fog 0.08" } },
						{330, new List<string> {"weather.fog 0.1" } },
						{350, new List<string> {"weather.fog 0.12" } },
						{370, new List<string> {"weather.fog 0.14" } },
						{390, new List<string> {"weather.fog 0.16" } },
						{410, new List<string> {"weather.fog 0.18", "weather.atmosphere_rayleigh 3.0" } },
						{430, new List<string> {"weather.fog 0.20" } },
						{450, new List<string> {"weather.fog 0.22" } },
						{470, new List<string> {"weather.fog 0.24" } },
						{490, new List<string> {"weather.fog 0.26" } },
						{510, new List<string> {"weather.fog 0.28" } },
						{530, new List<string> {"weather.fog 0.3", "weather.atmosphere_rayleigh 4.0" } },
						{2560, new List<string> {"weather.fog 0.28" } },
						{2580, new List<string> {"weather.fog 0.26" } },
						{2600, new List<string> {"weather.fog 0.24" } },
						{2620, new List<string> {"weather.fog 0.22" } },
						{2640, new List<string> {"weather.fog 0.2" } },
						{2660, new List<string> {"weather.fog 0.18", "weather.atmosphere_rayleigh 3.0" } },
						{2680, new List<string> {"weather.fog 0.16" } },
						{2700, new List<string> {"weather.fog 0.14" } },
						{2720, new List<string> {"weather.fog 0.12" } },
						{2740, new List<string> {"weather.fog 0.1" } },
						{2760, new List<string> {"weather.fog 0.08" } },
						{2780, new List<string> {"weather.fog 0.06" } },
						{2800, new List<string> {"weather.fog 0.04", "weather.atmosphere_rayleigh 2.0" } },
						{2820, new List<string> {"weather.fog 0.02" } },
						{2830, new List<string> {"weather.cloud_coverage 0.5" } },
						{2840, new List<string> {"weather.fog 0" } },
						{2850, new List<string> {"weather.cloud_coverage 0.48" } },
						{2860, new List<string> {"weather.cloud_coverage 0.46" } },
						{2870, new List<string> {"weather.cloud_coverage 0.44" } },
						{2880, new List<string> {"weather.cloud_coverage 0.42" } },
						{2890, new List<string> {"weather.cloud_coverage 0.4", "weather.atmosphere_rayleigh 1.5", "weather.atmosphere_contrast 1.5" } },
						{2900, new List<string> {"weather.cloud_coverage 0.38" } },
						{2910, new List<string> {"weather.cloud_coverage 0.36" } },
						{2920, new List<string> {"weather.cloud_coverage 0.34" } },
						{2930, new List<string> {"weather.cloud_coverage 0.32" } },
						{2940, new List<string> {"weather.cloud_coverage 0.3", "weather.atmosphere_rayleigh -1", "weather.atmosphere_contrast -1", "weather.cloud_size 0.8" } },
						{2950, new List<string> {"weather.cloud_coverage 0.28" } },
						{2960, new List<string> {"weather.cloud_coverage 0.26" } },
						{2970, new List<string> {"weather.cloud_coverage 0.24" } },
						{2980, new List<string> {"weather.cloud_coverage 0.22" } },
						{2990, new List<string> {"weather.cloud_coverage 0.2", "weather.cloud_size 0.9" } },
						{3000, new List<string> {"weather.cloud_coverage 0.18" } },
						{3010, new List<string> {"weather.cloud_coverage 0.16" } },
						{3020, new List<string> {"weather.cloud_coverage 0.14" } },
						{3030, new List<string> {"weather.cloud_coverage 0.12" } },
						{3040, new List<string> {"weather.cloud_coverage 0.1", "weather.cloud_size -1" } },
						{3050, new List<string> {"weather.cloud_coverage 0.08" } },
						{3060, new List<string> {"weather.cloud_coverage 0.06" } },
						{3070, new List<string> {"weather.cloud_coverage 0.04" } },
						{3080, new List<string> {"weather.cloud_coverage 0.02" } },						
                        {3090, new List<string> {"weather.cloud_coverage 0", "weather.reset", "STOP" } },
                    }
                    },
					{"heavyfog", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.cloud_coverage 0.02" } },
						{10, new List<string> {"weather.cloud_coverage 0.04" } },
						{20, new List<string> {"weather.cloud_coverage 0.06" } },
						{30, new List<string> {"weather.cloud_coverage 0.08" } },
						{40, new List<string> {"weather.cloud_coverage 0.1" } },
						{50, new List<string> {"weather.cloud_coverage 0.12" } },
						{60, new List<string> {"weather.cloud_coverage 0.14" } },
						{70, new List<string> {"weather.cloud_coverage 0.16" } },
						{80, new List<string> {"weather.cloud_coverage 0.18" } },
						{90, new List<string> {"weather.cloud_coverage 0.2" } },
						{100, new List<string> {"weather.cloud_coverage 0.22" } },
						{110, new List<string> {"weather.cloud_coverage 0.24" } },
						{120, new List<string> {"weather.cloud_coverage 0.26" } },
						{130, new List<string> {"weather.cloud_coverage 0.28" } },
						{140, new List<string> {"weather.cloud_coverage 0.3", "weather.cloud_attenuation 1.125", "weather.atmosphere_contrast 0.9", "weather.cloud_coloring 0.8" } },
						{150, new List<string> {"weather.cloud_coverage 0.32" } },
						{160, new List<string> {"weather.cloud_coverage 0.34" } },
						{170, new List<string> {"weather.cloud_coverage 0.36" } },
						{180, new List<string> {"weather.cloud_coverage 0.38" } },
						{190, new List<string> {"weather.cloud_coverage 0.4" } },
						{200, new List<string> {"weather.cloud_coverage 0.42" } },
						{210, new List<string> {"weather.cloud_coverage 0.44" } },
						{220, new List<string> {"weather.cloud_coverage 0.46" } },
						{230, new List<string> {"weather.cloud_coverage 0.48" } },
						{240, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_attenuation 1.25", "weather.atmosphere_contrast 0.8", "weather.cloud_coloring 0.6" } },
						{250, new List<string> {"weather.fog 0.02" } },
						{260, new List<string> {"weather.cloud_coverage 0.52" } },
						{270, new List<string> {"weather.fog 0.04" } },
						{280, new List<string> {"weather.cloud_coverage 0.54" } },
						{290, new List<string> {"weather.fog 0.06" } },
						{300, new List<string> {"weather.cloud_coverage 0.56" } },
						{310, new List<string> {"weather.fog 0.08" } },
						{320, new List<string> {"weather.cloud_coverage 0.58" } },
						{330, new List<string> {"weather.fog 0.1" } },
						{340, new List<string> {"weather.cloud_coverage 0.6" } },
						{350, new List<string> {"weather.fog 0.12" } },
						{360, new List<string> {"weather.cloud_coverage 0.62" } },
						{370, new List<string> {"weather.fog 0.14" } },
						{380, new List<string> {"weather.cloud_coverage 0.64" } },
						{390, new List<string> {"weather.fog 0.16" } },
						{400, new List<string> {"weather.cloud_coverage 0.66" } },
						{410, new List<string> {"weather.fog 0.18" } },
						{420, new List<string> {"weather.cloud_coverage 0.68" } },
						{430, new List<string> {"weather.fog 0.20" } },
						{440, new List<string> {"weather.cloud_coverage 0.7" } },
						{450, new List<string> {"weather.fog 0.22" } },
						{460, new List<string> {"weather.cloud_coverage 0.72" } },
						{470, new List<string> {"weather.fog 0.24" } },
						{480, new List<string> {"weather.cloud_coverage 0.74", "weather.cloud_attenuation 1.5", "weather.atmosphere_contrast 0.4", "weather.cloud_coloring 0.4" } },
						{490, new List<string> {"weather.fog 0.26" } },
						{500, new List<string> {"weather.cloud_coverage 0.76" } },
						{510, new List<string> {"weather.fog 0.28" } },
						{520, new List<string> {"weather.cloud_coverage 0.78" } },
						{530, new List<string> {"weather.fog 0.3" } },
						{540, new List<string> {"weather.cloud_coverage 0.8" } },
						{550, new List<string> {"weather.fog 0.32" } },
						{560, new List<string> {"weather.cloud_coverage 0.82" } },
						{570, new List<string> {"weather.fog 0.34" } },
						{580, new List<string> {"weather.cloud_coverage 0.84" } },
						{590, new List<string> {"weather.fog 0.36" } },
						{600, new List<string> {"weather.cloud_coverage 0.86" } },
						{610, new List<string> {"weather.fog 0.38" } },
						{620, new List<string> {"weather.cloud_coverage 0.88" } },
						{630, new List<string> {"weather.fog 0.4" } },
						{640, new List<string> {"weather.cloud_coverage 0.9" } },
						{650, new List<string> {"weather.fog 0.42" } },
						{660, new List<string> {"weather.cloud_coverage 0.92" } },
						{670, new List<string> {"weather.fog 0.44" } },
						{680, new List<string> {"weather.cloud_coverage 0.94" } },
						{690, new List<string> {"weather.fog 0.46" } },
						{700, new List<string> {"weather.cloud_coverage 0.96" } },
						{710, new List<string> {"weather.fog 0.48" } },
						{720, new List<string> {"weather.cloud_coverage 0.98" } },
						{730, new List<string> {"weather.fog 0.5" } },
						{740, new List<string> {"weather.cloud_coverage 1" } },
						{750, new List<string> {"weather.fog 0.52" } },
						{760, new List<string> {"weather.fog 0.54" } },
						{780, new List<string> {"weather.fog 0.56" } },
						{790, new List<string> {"weather.fog 0.58" } },
						{800, new List<string> {"weather.fog 0.6" } },
						{810, new List<string> {"weather.fog 0.62" } },
						{820, new List<string> {"weather.fog 0.64" } },
						{830, new List<string> {"weather.fog 0.66" } },
						{840, new List<string> {"weather.fog 0.68" } },
						{850, new List<string> {"weather.fog 0.7" } },
						{860, new List<string> {"weather.fog 0.72" } },
						{870, new List<string> {"weather.fog 0.74" } },
						{880, new List<string> {"weather.fog 0.76" } },
						{2410, new List<string> {"weather.fog 0.76" } },
						{2420, new List<string> {"weather.fog 0.74" } },
						{2430, new List<string> {"weather.fog 0.72" } },
						{2440, new List<string> {"weather.fog 0.7" } },
						{2450, new List<string> {"weather.fog 0.68" } },
						{2460, new List<string> {"weather.fog 0.66" } },
						{2470, new List<string> {"weather.fog 0.64" } },
						{2480, new List<string> {"weather.fog 0.62" } },
						{2490, new List<string> {"weather.fog 0.6" } },
						{2500, new List<string> {"weather.fog 0.58" } },
						{2510, new List<string> {"weather.fog 0.56" } },
						{2520, new List<string> {"weather.fog 0.54" } },
						{2530, new List<string> {"weather.fog 0.52" } },
						{2540, new List<string> {"weather.fog 0.5" } },
						{2550, new List<string> {"weather.cloud_coverage 0.98" } },
						{2560, new List<string> {"weather.fog 0.48" } },
						{2570, new List<string> {"weather.cloud_coverage 0.96" } },
						{2580, new List<string> {"weather.fog 0.46" } },
						{2590, new List<string> {"weather.cloud_coverage 0.94" } },
						{2600, new List<string> {"weather.fog 0.44" } },
						{2610, new List<string> {"weather.cloud_coverage 0.92" } },
						{2620, new List<string> {"weather.fog 0.42" } },
						{2630, new List<string> {"weather.cloud_coverage 0.9" } },
						{2640, new List<string> {"weather.fog 0.4" } },
						{2650, new List<string> {"weather.cloud_coverage 0.88" } },
						{2660, new List<string> {"weather.fog 0.38" } },
						{2670, new List<string> {"weather.cloud_coverage 0.86" } },
						{2680, new List<string> {"weather.fog 0.36" } },
						{2690, new List<string> {"weather.cloud_coverage 0.84" } },
						{2700, new List<string> {"weather.fog 0.34" } },
						{2710, new List<string> {"weather.cloud_coverage 0.82" } },
						{2720, new List<string> {"weather.fog 0.32" } },
						{2730, new List<string> {"weather.cloud_coverage 0.8" } },
						{2740, new List<string> {"weather.fog 0.3" } },
						{2750, new List<string> {"weather.cloud_coverage 0.78" } },
						{2760, new List<string> {"weather.fog 0.28" } },
						{2770, new List<string> {"weather.cloud_coverage 0.76", "weather.cloud_attenuation 1.25", "weather.atmosphere_contrast 0.8", "weather.cloud_coloring 0.6" } },
						{2780, new List<string> {"weather.fog 0.26" } },
						{2790, new List<string> {"weather.cloud_coverage 0.74" } },
						{2800, new List<string> {"weather.fog 0.24" } },
						{2810, new List<string> {"weather.cloud_coverage 0.72" } },
						{2820, new List<string> {"weather.fog 0.22" } },
						{2830, new List<string> {"weather.cloud_coverage 0.7" } },
						{2840, new List<string> {"weather.fog 0.2" } },
						{2850, new List<string> {"weather.cloud_coverage 0.68" } },
						{2860, new List<string> {"weather.fog 0.18" } },
						{2870, new List<string> {"weather.cloud_coverage 0.66" } },
						{2880, new List<string> {"weather.fog 0.16" } },
						{2890, new List<string> {"weather.cloud_coverage 0.64" } },
						{2900, new List<string> {"weather.fog 0.14" } },
						{2910, new List<string> {"weather.cloud_coverage 0.62" } },
						{2920, new List<string> {"weather.fog 0.12" } },
						{2930, new List<string> {"weather.cloud_coverage 0.6" } },
						{2940, new List<string> {"weather.fog 0.1" } },
						{2950, new List<string> {"weather.cloud_coverage 0.58" } },
						{2960, new List<string> {"weather.fog 0.08" } },
						{2970, new List<string> {"weather.cloud_coverage 0.56" } },
						{2980, new List<string> {"weather.fog 0.06" } },
						{2990, new List<string> {"weather.cloud_coverage 0.54" } },
						{3000, new List<string> {"weather.fog 0.04" } },
						{3010, new List<string> {"weather.cloud_coverage 0.52" } },
						{3020, new List<string> {"weather.fog 0.02" } },
						{3030, new List<string> {"weather.cloud_coverage 0.5" } },
						{3040, new List<string> {"weather.fog 0" } },
						{3050, new List<string> {"weather.cloud_coverage 0.48" } },
						{3060, new List<string> {"weather.cloud_coverage 0.46" } },
						{3070, new List<string> {"weather.cloud_coverage 0.44" } },
						{3080, new List<string> {"weather.cloud_coverage 0.42" } },
						{3090, new List<string> {"weather.cloud_coverage 0.4" } },
						{3100, new List<string> {"weather.cloud_coverage 0.38" } },
						{3110, new List<string> {"weather.cloud_coverage 0.36" } },
						{3120, new List<string> {"weather.cloud_coverage 0.34", "weather.cloud_attenuation 0.125", "weather.atmosphere_contrast 0.9", "weather.cloud_coloring 0.8" } },
						{3130, new List<string> {"weather.cloud_coverage 0.32" } },
						{3140, new List<string> {"weather.cloud_coverage 0.3" } },
						{3150, new List<string> {"weather.cloud_coverage 0.28" } },
						{3160, new List<string> {"weather.cloud_coverage 0.26" } },
						{3170, new List<string> {"weather.cloud_coverage 0.24" } },
						{3180, new List<string> {"weather.cloud_coverage 0.22" } },
						{3190, new List<string> {"weather.cloud_coverage 0.2", "weather.cloud_attenuation -1", "weather.atmosphere_contrast -1", "weather.cloud_coloring -1" } },
						{3200, new List<string> {"weather.cloud_coverage 0.18" } },
						{3210, new List<string> {"weather.cloud_coverage 0.16" } },
						{3220, new List<string> {"weather.cloud_coverage 0.14" } },
						{3230, new List<string> {"weather.cloud_coverage 0.12" } },
						{3240, new List<string> {"weather.cloud_coverage 0.1" } },
						{3250, new List<string> {"weather.cloud_coverage 0.08" } },
						{3260, new List<string> {"weather.cloud_coverage 0.06" } },
						{3270, new List<string> {"weather.cloud_coverage 0.04" } },
						{3280, new List<string> {"weather.cloud_coverage 0.02" } },						
                        {3290, new List<string> {"weather.cloud_coverage 0", "weather.reset", "STOP" } },
                    }
                    },
					{"overcast", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.cloud_coverage 0.01" } },
						{10, new List<string> {"weather.cloud_coverage 0.02" } },
						{20, new List<string> {"weather.cloud_coverage 0.04" } },
						{30, new List<string> {"weather.cloud_coverage 0.06" } },
						{40, new List<string> {"weather.cloud_coverage 0.08" } },
						{50, new List<string> {"weather.cloud_coverage 0.1" } },
						{60, new List<string> {"weather.cloud_coverage 0.12" } },
						{70, new List<string> {"weather.cloud_coverage 0.14" } },
						{80, new List<string> {"weather.cloud_coverage 0.16" } },
						{90, new List<string> {"weather.cloud_coverage 0.18" } },
						{100, new List<string> {"weather.cloud_coverage 0.2" } },
						{110, new List<string> {"weather.cloud_coverage 0.22" } },
						{120, new List<string> {"weather.cloud_coverage 0.24" } },
						{130, new List<string> {"weather.cloud_coverage 0.26" } },
						{140, new List<string> {"weather.cloud_coverage 0.28" } },
						{150, new List<string> {"weather.cloud_coverage 0.3" } },
						{160, new List<string> {"weather.cloud_coverage 0.32", "weather.cloud_attenuation 0.1", "weather.atmosphere_contrast 0.9" } },
						{170, new List<string> {"weather.cloud_coverage 0.34" } },
						{180, new List<string> {"weather.cloud_coverage 0.36" } },
						{190, new List<string> {"weather.cloud_coverage 0.38" } },
						{200, new List<string> {"weather.cloud_coverage 0.4" } },
						{210, new List<string> {"weather.cloud_coverage 0.42" } },
						{220, new List<string> {"weather.cloud_coverage 0.44" } },
						{230, new List<string> {"weather.cloud_coverage 0.46" } },
						{240, new List<string> {"weather.cloud_coverage 0.48" } },
						{250, new List<string> {"weather.cloud_coverage 0.5" } },
						{260, new List<string> {"weather.cloud_coverage 0.52" } },
						{270, new List<string> {"weather.cloud_coverage 0.54" } },
						{280, new List<string> {"weather.cloud_coverage 0.56" } },
						{290, new List<string> {"weather.cloud_coverage 0.58" } },
						{300, new List<string> {"weather.cloud_coverage 0.6" } },
						{310, new List<string> {"weather.cloud_coverage 0.62" } },
						{320, new List<string> {"weather.cloud_coverage 0.64" } },
						{330, new List<string> {"weather.cloud_coverage 0.66", "weather.cloud_attenuation 0.2", "weather.atmosphere_contrast 0.8" } },
						{340, new List<string> {"weather.cloud_coverage 0.68" } },
						{350, new List<string> {"weather.cloud_coverage 0.7" } },
						{360, new List<string> {"weather.cloud_coverage 0.72" } },
						{370, new List<string> {"weather.cloud_coverage 0.74" } },
						{380, new List<string> {"weather.cloud_coverage 0.76" } },
						{390, new List<string> {"weather.cloud_coverage 0.78" } },
						{400, new List<string> {"weather.cloud_coverage 0.8" } },
						{410, new List<string> {"weather.cloud_coverage 0.82" } },
						{420, new List<string> {"weather.cloud_coverage 0.84" } },
						{430, new List<string> {"weather.cloud_coverage 0.86" } },
						{440, new List<string> {"weather.cloud_coverage 0.88" } },
						{450, new List<string> {"weather.cloud_coverage 0.9" } },
						{460, new List<string> {"weather.cloud_coverage 0.92" } },
						{470, new List<string> {"weather.cloud_coverage 0.94" } },
						{480, new List<string> {"weather.cloud_coverage 0.96" } },
						{490, new List<string> {"weather.cloud_coverage 0.98" } },
						{500, new List<string> {"weather.cloud_coverage 1" } },
						{2500, new List<string> {"weather.cloud_coverage 0.98" } },
						{2510, new List<string> {"weather.cloud_coverage 0.96" } },
						{2520, new List<string> {"weather.cloud_coverage 0.94" } },
						{2530, new List<string> {"weather.cloud_coverage 0.92" } },
						{2640, new List<string> {"weather.cloud_coverage 0.9" } },
						{2650, new List<string> {"weather.cloud_coverage 0.88" } },
						{2660, new List<string> {"weather.cloud_coverage 0.86" } },
						{2670, new List<string> {"weather.cloud_coverage 0.84" } },
						{2780, new List<string> {"weather.cloud_coverage 0.82" } },
						{2790, new List<string> {"weather.cloud_coverage 0.8" } },
						{2700, new List<string> {"weather.cloud_coverage 0.78" } },
						{2710, new List<string> {"weather.cloud_coverage 0.76" } },
						{2820, new List<string> {"weather.cloud_coverage 0.74" } },
						{2830, new List<string> {"weather.cloud_coverage 0.72" } },
						{2840, new List<string> {"weather.cloud_coverage 0.7" } },
						{2850, new List<string> {"weather.cloud_coverage 0.68" } },
						{2960, new List<string> {"weather.cloud_coverage 0.66", "weather.cloud_attenuation 0.1", "weather.atmosphere_contrast 0.9" } },
						{2970, new List<string> {"weather.cloud_coverage 0.64" } },
						{2980, new List<string> {"weather.cloud_coverage 0.62" } },
						{2990, new List<string> {"weather.cloud_coverage 0.6" } },
						{3300, new List<string> {"weather.cloud_coverage 0.58" } },
						{3310, new List<string> {"weather.cloud_coverage 0.56" } },
						{3320, new List<string> {"weather.cloud_coverage 0.54" } },
						{3330, new List<string> {"weather.cloud_coverage 0.52" } },
						{3340, new List<string> {"weather.cloud_coverage 0.5" } },
						{3350, new List<string> {"weather.cloud_coverage 0.48" } },
						{3360, new List<string> {"weather.cloud_coverage 0.46" } },
						{3370, new List<string> {"weather.cloud_coverage 0.44" } },
						{3380, new List<string> {"weather.cloud_coverage 0.42" } },
						{3390, new List<string> {"weather.cloud_coverage 0.4" } },
						{3400, new List<string> {"weather.cloud_coverage 0.38" } },
						{3410, new List<string> {"weather.cloud_coverage 0.36" } },
						{3420, new List<string> {"weather.cloud_coverage 0.34" } },
						{3430, new List<string> {"weather.cloud_coverage 0.32", "weather.cloud_attenuation -1", "weather.atmosphere_contrast -1" } },
						{3440, new List<string> {"weather.cloud_coverage 0.3" } },
						{3450, new List<string> {"weather.cloud_coverage 0.28" } },
						{3460, new List<string> {"weather.cloud_coverage 0.26" } },
						{3470, new List<string> {"weather.cloud_coverage 0.24" } },
						{3480, new List<string> {"weather.cloud_coverage 0.22" } },
						{3490, new List<string> {"weather.cloud_coverage 0.2" } },
						{3500, new List<string> {"weather.cloud_coverage 0.18" } },
						{3510, new List<string> {"weather.cloud_coverage 0.16" } },
						{3520, new List<string> {"weather.cloud_coverage 0.14" } },
						{3530, new List<string> {"weather.cloud_coverage 0.12" } },
						{3540, new List<string> {"weather.cloud_coverage 0.1" } },
						{3550, new List<string> {"weather.cloud_coverage 0.08" } },
						{3560, new List<string> {"weather.cloud_coverage 0.06" } },
						{3570, new List<string> {"weather.cloud_coverage 0.04" } },
						{3580, new List<string> {"weather.cloud_coverage 0.02" } },
                        {3590, new List<string> {"weather.cloud_coverage 0", "weather.reset", "STOP" } },
                    }
                    },
					{"partlycloudy", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.cloud_coverage 0.02" } },
						{10, new List<string> {"weather.cloud_coverage 0.04" } },
						{20, new List<string> {"weather.cloud_coverage 0.06" } },
						{30, new List<string> {"weather.cloud_coverage 0.08" } },
						{40, new List<string> {"weather.cloud_coverage 0.1", "weather.cloud_opacity 0.9", "weather.cloud_size 0.9" } },
						{50, new List<string> {"weather.cloud_coverage 0.12" } },
						{60, new List<string> {"weather.cloud_coverage 0.14" } },
						{70, new List<string> {"weather.cloud_coverage 0.16" } },
						{80, new List<string> {"weather.cloud_coverage 0.18" } },
						{90, new List<string> {"weather.cloud_coverage 0.2", "weather.cloud_opacity 0.8" } },
						{100, new List<string> {"weather.cloud_coverage 0.22" } },
						{110, new List<string> {"weather.cloud_coverage 0.24" } },
						{120, new List<string> {"weather.cloud_coverage 0.26" } },
						{130, new List<string> {"weather.cloud_coverage 0.28" } },
						{140, new List<string> {"weather.cloud_coverage 0.3", "weather.cloud_opacity 0.7" } },
						{150, new List<string> {"weather.cloud_coverage 0.32" } },
						{160, new List<string> {"weather.cloud_coverage 0.34" } },
						{170, new List<string> {"weather.cloud_coverage 0.36" } },
						{180, new List<string> {"weather.cloud_coverage 0.38" } },
						{190, new List<string> {"weather.cloud_coverage 0.4" } },
						{300, new List<string> {"weather.cloud_coverage 0.42" } },
						{310, new List<string> {"weather.cloud_coverage 0.44" } },
						{320, new List<string> {"weather.cloud_coverage 0.46" } },
						{330, new List<string> {"weather.cloud_coverage 0.48" } },
						{340, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_attenuation 0.1" } },
						{360, new List<string> {"weather.cloud_coverage 0.52" } },
						{380, new List<string> {"weather.cloud_coverage 0.54" } },
						{500, new List<string> {"weather.cloud_coverage 0.56" } },
						{520, new List<string> {"weather.cloud_coverage 0.58" } },
						{540, new List<string> {"weather.cloud_coverage 0.6" } },
						{560, new List<string> {"weather.cloud_coverage 0.62" } },
						{580, new List<string> {"weather.cloud_coverage 0.64" } },
						{700, new List<string> {"weather.cloud_coverage 0.66" } },
						{720, new List<string> {"weather.cloud_coverage 0.68" } },
						{740, new List<string> {"weather.cloud_coverage 0.7" } },
						{760, new List<string> {"weather.cloud_coverage 0.72" } },
						{780, new List<string> {"weather.cloud_coverage 0.74" } },
						{900, new List<string> {"weather.cloud_coverage 0.75" } },
						{2290, new List<string> {"weather.cloud_coverage 0.74" } },
						{2310, new List<string> {"weather.cloud_coverage 0.72" } },
						{2330, new List<string> {"weather.cloud_coverage 0.7" } },
						{2350, new List<string> {"weather.cloud_coverage 0.68" } },
						{2370, new List<string> {"weather.cloud_coverage 0.66" } },
						{2390, new List<string> {"weather.cloud_coverage 0.64" } },
						{2510, new List<string> {"weather.cloud_coverage 0.62" } },
						{2530, new List<string> {"weather.cloud_coverage 0.6" } },
						{2550, new List<string> {"weather.cloud_coverage 0.58" } },
						{2570, new List<string> {"weather.cloud_coverage 0.56" } },
						{2590, new List<string> {"weather.cloud_coverage 0.54" } },
						{2710, new List<string> {"weather.cloud_coverage 0.52" } },
						{2730, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_attenuation -1" } },
						{2750, new List<string> {"weather.cloud_coverage 0.48" } },
						{2760, new List<string> {"weather.cloud_coverage 0.46" } },
						{2770, new List<string> {"weather.cloud_coverage 0.44" } },
						{2780, new List<string> {"weather.cloud_coverage 0.42" } },
						{2790, new List<string> {"weather.cloud_coverage 0.4" } },
						{2900, new List<string> {"weather.cloud_coverage 0.38" } },
						{2910, new List<string> {"weather.cloud_coverage 0.36" } },
						{2920, new List<string> {"weather.cloud_coverage 0.34" } },
						{2930, new List<string> {"weather.cloud_coverage 0.32" } },
						{2940, new List<string> {"weather.cloud_coverage 0.3", "weather.cloud_opacity 0.8" } },
						{2950, new List<string> {"weather.cloud_coverage 0.28" } },
						{2960, new List<string> {"weather.cloud_coverage 0.26" } },
						{2970, new List<string> {"weather.cloud_coverage 0.24" } },
						{2980, new List<string> {"weather.cloud_coverage 0.22" } },
						{2990, new List<string> {"weather.cloud_coverage 0.2", "weather.cloud_opacity 0.9" } },
						{3100, new List<string> {"weather.cloud_coverage 0.18" } },
						{3110, new List<string> {"weather.cloud_coverage 0.16" } },
						{3120, new List<string> {"weather.cloud_coverage 0.14" } },
						{3130, new List<string> {"weather.cloud_coverage 0.12" } },
						{3140, new List<string> {"weather.cloud_coverage 0.1", "weather.cloud_opacity -1" } },
						{3150, new List<string> {"weather.cloud_coverage 0.08" } },
						{3160, new List<string> {"weather.cloud_coverage 0.06" } },
						{3170, new List<string> {"weather.cloud_coverage 0.04" } },
						{3180, new List<string> {"weather.cloud_coverage 0.02" } },						
                        {3190, new List<string> {"weather.cloud_coverage 0", "weather.reset", "STOP" } },
                    }
                    },
					{"spottyshowers", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.cloud_coverage 0.01" } },
						{10, new List<string> {"weather.cloud_coverage 0.02" } },
						{20, new List<string> {"weather.cloud_coverage 0.04", "weather.cloud_size 1.25" } },
						{30, new List<string> {"weather.cloud_coverage 0.06" } },
						{40, new List<string> {"weather.cloud_coverage 0.08" } },
						{50, new List<string> {"weather.cloud_coverage 0.1", "weather.cloud_size 1.5" } },
						{60, new List<string> {"weather.cloud_coverage 0.12" } },
						{70, new List<string> {"weather.cloud_coverage 0.14" } },
						{80, new List<string> {"weather.cloud_coverage 0.16" } },
						{90, new List<string> {"weather.cloud_coverage 0.18" } },
						{100, new List<string> {"weather.cloud_coverage 0.2" } },
						{110, new List<string> {"weather.cloud_coverage 0.22" } },
						{120, new List<string> {"weather.cloud_coverage 0.24" } },
						{130, new List<string> {"weather.cloud_coverage 0.26" } },
						{140, new List<string> {"weather.cloud_coverage 0.28" } },
						{150, new List<string> {"weather.cloud_coverage 0.3" } },
						{160, new List<string> {"weather.cloud_coverage 0.32" } },
						{170, new List<string> {"weather.cloud_coverage 0.34" } },
						{180, new List<string> {"weather.cloud_coverage 0.36" } },
						{190, new List<string> {"weather.cloud_coverage 0.38" } },
						{200, new List<string> {"weather.cloud_coverage 0.4", "weather.atmosphere_rayleigh 1.5" } },
						{210, new List<string> {"weather.cloud_coverage 0.42" } },
						{220, new List<string> {"weather.cloud_coverage 0.44" } },
						{230, new List<string> {"weather.cloud_coverage 0.46" } },
						{240, new List<string> {"weather.cloud_coverage 0.48" } },
						{250, new List<string> {"weather.cloud_coverage 0.5" } },
						{260, new List<string> {"weather.cloud_coverage 0.52" } },
						{270, new List<string> {"weather.cloud_coverage 0.54" } },
						{280, new List<string> {"weather.cloud_coverage 0.56" } },
						{290, new List<string> {"weather.cloud_coverage 0.58" } },
						{300, new List<string> {"weather.cloud_coverage 0.6" } },
						{310, new List<string> {"weather.cloud_coverage 0.62" } },
						{320, new List<string> {"weather.cloud_coverage 0.64" } },
						{330, new List<string> {"weather.cloud_coverage 0.66" } },
						{340, new List<string> {"weather.cloud_coverage 0.68" } },
						{350, new List<string> {"weather.cloud_coverage 0.7" } },
						{360, new List<string> {"weather.cloud_coverage 0.72" } },
						{370, new List<string> {"weather.cloud_coverage 0.74" } },
						{380, new List<string> { "weather.cloud_coverage 0.75", "weather.cloud_attenuation 0.1" } },
                        {480, new List<string> { "weather.rain 0.03", "weather.rainbow 1" } },
                        {510, new List<string> { "weather.rain 0.06" } },
                        {560, new List<string> { "weather.rain 0.09" } },
                        {590, new List<string> { "weather.rain 0.12" } },
                        {620, new List<string> { "weather.rain 0.15" } },
                        {1090, new List<string> { "weather.rain 0.18" } },
                        {1150, new List<string> { "weather.rain 0.21" } },
						{1180, new List<string> { "weather.cloud_coverage 0.74", "weather.cloud_attenuation -1" } },
						{1190, new List<string> { "weather.cloud_coverage 0.72" } },
						{1200, new List<string> { "weather.cloud_coverage 0.7" } },
                        {1210, new List<string> { "weather.rain 0.15" } },
						{1220, new List<string> { "weather.cloud_coverage 0.68" } },
						{1230, new List<string> { "weather.cloud_coverage 0.66" } },
						{1240, new List<string> { "weather.cloud_coverage 0.64" } },
						{1250, new List<string> { "weather.cloud_coverage 0.62" } },
                        {1260, new List<string> { "weather.rain 0.08" } },
						{1270, new List<string> { "weather.cloud_coverage 0.6" } },
						{1280, new List<string> { "weather.cloud_coverage 0.58" } },
						{1290, new List<string> { "weather.cloud_coverage 0.56" } },
						{1300, new List<string> { "weather.cloud_coverage 0.54" } },
						{1310, new List<string> { "weather.cloud_coverage 0.52" } },
						{1320, new List<string> { "weather.cloud_coverage 0.5" } },
                        {1330, new List<string> { "weather.rain 0" } },
						{1350, new List<string> { "weather.cloud_coverage 0.48" } },
						{1370, new List<string> { "weather.cloud_coverage 0.46" } },
						{1390, new List<string> { "weather.cloud_coverage 0.44" } },
						{1410, new List<string> { "weather.cloud_coverage 0.42" } },
						{1430, new List<string> { "weather.cloud_coverage 0.4", "weather.atmosphere_rayleigh -1" } },
						{1450, new List<string> { "weather.cloud_coverage 0.38" } },
						{1470, new List<string> { "weather.cloud_coverage 0.36" } },
						{1490, new List<string> { "weather.cloud_coverage 0.34" } },
						{1510, new List<string> { "weather.cloud_coverage 0.32" } },
						{2500, new List<string> {"weather.cloud_coverage 0.3" } },
						{2510, new List<string> {"weather.cloud_coverage 0.32" } },
						{2520, new List<string> {"weather.cloud_coverage 0.34" } },
						{2530, new List<string> {"weather.cloud_coverage 0.36" } },
						{2540, new List<string> {"weather.cloud_coverage 0.38" } },
						{2550, new List<string> {"weather.cloud_coverage 0.4", "weather.atmosphere_rayleigh 1.5" } },
						{2560, new List<string> {"weather.cloud_coverage 0.42" } },
						{2570, new List<string> {"weather.cloud_coverage 0.44" } },
						{2580, new List<string> {"weather.cloud_coverage 0.46" } },
						{2590, new List<string> {"weather.cloud_coverage 0.48" } },
						{3000, new List<string> {"weather.cloud_coverage 0.5" } },
						{3010, new List<string> {"weather.cloud_coverage 0.52" } },
						{3020, new List<string> {"weather.cloud_coverage 0.54" } },
						{3030, new List<string> {"weather.cloud_coverage 0.56" } },
						{3040, new List<string> {"weather.cloud_coverage 0.58" } },
						{3050, new List<string> {"weather.cloud_coverage 0.6" } },
						{3060, new List<string> {"weather.cloud_coverage 0.64" } },
						{3070, new List<string> {"weather.cloud_coverage 0.66" } },
						{3080, new List<string> {"weather.cloud_coverage 0.68" } },
						{3090, new List<string> {"weather.cloud_coverage 0.7" } },
						{3100, new List<string> {"weather.cloud_coverage 0.72" } },
						{3110, new List<string> {"weather.cloud_coverage 0.74" } },
						{3120, new List<string> { "weather.cloud_coverage 0.75", "weather.cloud_attenuation 0.1" } },
                        {3130, new List<string> { "weather.rain 0.03" } },
                        {3140, new List<string> { "weather.rain 0.06" } },
                        {3150, new List<string> { "weather.rain 0.09" } },
                        {3160, new List<string> { "weather.rain 0.12" } },
                        {3170, new List<string> { "weather.rain 0.15" } },
                        {3180, new List<string> { "weather.rain 0.18" } },
                        {3190, new List<string> { "weather.rain 0.21" } },
						{3200, new List<string> { "weather.cloud_coverage 0.74", "weather.cloud_attenuation -1" } },
						{3205, new List<string> { "weather.cloud_coverage 0.72" } },
						{3210, new List<string> { "weather.cloud_coverage 0.7" } },
                        {3215, new List<string> { "weather.rain 0.15" } },
						{3220, new List<string> { "weather.cloud_coverage 0.68" } },
						{3225, new List<string> { "weather.cloud_coverage 0.66" } },
						{3230, new List<string> { "weather.cloud_coverage 0.64" } },
						{3235, new List<string> { "weather.cloud_coverage 0.62" } },
                        {3240, new List<string> { "weather.rain 0.08" } },
						{3245, new List<string> { "weather.cloud_coverage 0.6" } },
						{3250, new List<string> { "weather.cloud_coverage 0.58" } },
						{3255, new List<string> { "weather.cloud_coverage 0.56" } },
						{3260, new List<string> { "weather.cloud_coverage 0.54" } },
						{3265, new List<string> { "weather.cloud_coverage 0.52" } },
						{3270, new List<string> { "weather.cloud_coverage 0.5" } },
                        {3280, new List<string> { "weather.rain 0" } },
						{3285, new List<string> { "weather.cloud_coverage 0.48" } },
						{3290, new List<string> { "weather.cloud_coverage 0.46" } },
						{3295, new List<string> { "weather.cloud_coverage 0.44" } },
						{3300, new List<string> { "weather.cloud_coverage 0.42" } },
						{3305, new List<string> { "weather.cloud_coverage 0.4", "weather.atmosphere_rayleigh -1" } },
						{3310, new List<string> { "weather.cloud_coverage 0.38" } },
						{3315, new List<string> { "weather.cloud_coverage 0.36" } },
						{3320, new List<string> { "weather.cloud_coverage 0.34" } },
						{3325, new List<string> { "weather.cloud_coverage 0.32", "weather.cloud_size 1.25" } },
						{3330, new List<string> { "weather.cloud_coverage 0.3" } },
						{3335, new List<string> { "weather.cloud_coverage 0.28" } },
						{3340, new List<string> { "weather.cloud_coverage 0.26" } },
						{3345, new List<string> { "weather.cloud_coverage 0.24" } },
						{3350, new List<string> { "weather.cloud_coverage 0.22" } },
						{3355, new List<string> { "weather.cloud_coverage 0.2" } },
						{3360, new List<string> { "weather.cloud_coverage 0.18" } },
						{3365, new List<string> { "weather.cloud_coverage 0.16" } },
						{3370, new List<string> { "weather.cloud_coverage 0.14" } },
						{3375, new List<string> { "weather.cloud_coverage 0.12" } },
						{3380, new List<string> { "weather.cloud_coverage 0.1", "weather.cloud_size -1" } },
						{3385, new List<string> { "weather.cloud_coverage 0.08" } },
						{3390, new List<string> { "weather.cloud_coverage 0.00", "weather.rainbow -1" } },
                        {3395, new List<string> {"weather.cloud_coverage 0", "weather.reset", "STOP" } },
                    }
                    },
					{"sunandclouds", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.cloud_coverage 0.02" } },
						{10, new List<string> {"weather.cloud_coverage 0.04" } },
						{20, new List<string> {"weather.cloud_coverage 0.06" } },
						{30, new List<string> {"weather.cloud_coverage 0.08" } },
						{40, new List<string> {"weather.cloud_coverage 0.1", "weather.cloud_opacity 0.9", "weather.cloud_size 0.9" } },
						{50, new List<string> {"weather.cloud_coverage 0.12" } },
						{60, new List<string> {"weather.cloud_coverage 0.14" } },
						{70, new List<string> {"weather.cloud_coverage 0.16" } },
						{80, new List<string> {"weather.cloud_coverage 0.18" } },
						{90, new List<string> {"weather.cloud_coverage 0.2", "weather.cloud_opacity 0.8", "weather.cloud_size 0.8" } },
						{100, new List<string> {"weather.cloud_coverage 0.22" } },
						{110, new List<string> {"weather.cloud_coverage 0.24" } },
						{120, new List<string> {"weather.cloud_coverage 0.26" } },
						{130, new List<string> {"weather.cloud_coverage 0.28" } },
						{140, new List<string> {"weather.cloud_coverage 0.3" } },
						{150, new List<string> {"weather.cloud_coverage 0.32" } },
						{160, new List<string> {"weather.cloud_coverage 0.34" } },
						{170, new List<string> {"weather.cloud_coverage 0.36" } },
						{180, new List<string> {"weather.cloud_coverage 0.38" } },
						{190, new List<string> {"weather.cloud_coverage 0.4" } },
						{300, new List<string> {"weather.cloud_coverage 0.42" } },
						{310, new List<string> {"weather.cloud_coverage 0.44" } },
						{320, new List<string> {"weather.cloud_coverage 0.46" } },
						{330, new List<string> {"weather.cloud_coverage 0.48" } },
						{340, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_attenuation 0.1" } },
						{1730, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_attenuation -1" } },
						{1750, new List<string> {"weather.cloud_coverage 0.48" } },
						{1760, new List<string> {"weather.cloud_coverage 0.46" } },
						{1770, new List<string> {"weather.cloud_coverage 0.44" } },
						{1780, new List<string> {"weather.cloud_coverage 0.42" } },
						{1790, new List<string> {"weather.cloud_coverage 0.4" } },
						{1900, new List<string> {"weather.cloud_coverage 0.38" } },
						{1910, new List<string> {"weather.cloud_coverage 0.36" } },
						{1920, new List<string> {"weather.cloud_coverage 0.34" } },
						{1930, new List<string> {"weather.cloud_coverage 0.32" } },
						{1940, new List<string> {"weather.cloud_coverage 0.3" } },
						{1950, new List<string> {"weather.cloud_coverage 0.28" } },
						{1960, new List<string> {"weather.cloud_coverage 0.26" } },
						{2320, new List<string> {"weather.cloud_coverage 0.28" } },
						{2330, new List<string> {"weather.cloud_coverage 0.3" } },
						{2340, new List<string> {"weather.cloud_coverage 0.32" } },
						{2350, new List<string> {"weather.cloud_coverage 0.34" } },
						{2360, new List<string> {"weather.cloud_coverage 0.36" } },
						{2370, new List<string> {"weather.cloud_coverage 0.38" } },
						{2380, new List<string> {"weather.cloud_coverage 0.4", "weather.atmosphere_rayleigh 1.25" } },
						{2390, new List<string> {"weather.cloud_coverage 0.42" } },
						{2400, new List<string> {"weather.cloud_coverage 0.44" } },
						{2410, new List<string> {"weather.cloud_coverage 0.46" } },
						{2420, new List<string> {"weather.cloud_coverage 0.48" } },
						{2430, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_attenuation 0.1" } },
						{2440, new List<string> {"weather.cloud_coverage 0.52" } },
						{2450, new List<string> {"weather.cloud_coverage 0.54" } },
						{2460, new List<string> {"weather.cloud_coverage 0.56" } },
						{2470, new List<string> {"weather.cloud_coverage 0.58" } },
						{2480, new List<string> {"weather.cloud_coverage 0.6" } },
						{2650, new List<string> {"weather.cloud_coverage 0.58" } },
						{2660, new List<string> {"weather.cloud_coverage 0.56" } },
						{2670, new List<string> {"weather.cloud_coverage 0.54" } },
						{2680, new List<string> {"weather.cloud_coverage 0.52" } },
						{2690, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_attenuation -1" } },
						{2700, new List<string> {"weather.cloud_coverage 0.48" } },
						{2710, new List<string> {"weather.cloud_coverage 0.46" } },
						{2720, new List<string> {"weather.cloud_coverage 0.44" } },
						{2730, new List<string> {"weather.cloud_coverage 0.42" } },
						{2740, new List<string> {"weather.cloud_coverage 0.4", "weather.atmosphere_rayleigh -1" } },
						{2750, new List<string> {"weather.cloud_coverage 0.38" } },
						{2760, new List<string> {"weather.cloud_coverage 0.36" } },
						{2770, new List<string> {"weather.cloud_coverage 0.34" } },
						{2930, new List<string> {"weather.cloud_coverage 0.32" } },
						{2940, new List<string> {"weather.cloud_coverage 0.3" } },
						{2950, new List<string> {"weather.cloud_coverage 0.28" } },
						{2960, new List<string> {"weather.cloud_coverage 0.26" } },
						{2970, new List<string> {"weather.cloud_coverage 0.24" } },
						{2980, new List<string> {"weather.cloud_coverage 0.22" } },
						{2990, new List<string> {"weather.cloud_coverage 0.2", "weather.cloud_opacity 0.9", "weather.cloud_size 0.9" } },
						{3100, new List<string> {"weather.cloud_coverage 0.18" } },
						{3110, new List<string> {"weather.cloud_coverage 0.16" } },
						{3120, new List<string> {"weather.cloud_coverage 0.14" } },
						{3130, new List<string> {"weather.cloud_coverage 0.12" } },
						{3140, new List<string> {"weather.cloud_coverage 0.1", "weather.cloud_opacity -1", "weather.cloud_size -1" } },
						{3150, new List<string> {"weather.cloud_coverage 0.08" } },
						{3160, new List<string> {"weather.cloud_coverage 0.06" } },
						{3170, new List<string> {"weather.cloud_coverage 0.04" } },
						{3180, new List<string> {"weather.cloud_coverage 0.02" } },								
                        {3190, new List<string> {"weather.cloud_coverage 0", "weather.reset", "STOP" } },
                    }
                    },
					{"quickfront", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.cloud_coverage 0.02" } },
						{10, new List<string> {"weather.cloud_coverage 0.04" } },
						{20, new List<string> {"weather.cloud_coverage 0.06" } },
						{30, new List<string> {"weather.cloud_coverage 0.08" } },
						{40, new List<string> {"weather.cloud_coverage 0.1", "weather.cloud_opacity 0.9", "weather.cloud_size 0.9" } },
						{50, new List<string> {"weather.cloud_coverage 0.12", "weather.wind 0.1" } },
						{60, new List<string> {"weather.cloud_coverage 0.14", "weather.wind 0.2" } },
						{70, new List<string> {"weather.cloud_coverage 0.16", "weather.wind 0.3" } },
						{80, new List<string> {"weather.cloud_coverage 0.18", "weather.wind 0.4" } },
						{90, new List<string> {"weather.cloud_coverage 0.2", "weather.cloud_opacity 0.8", "weather.cloud_size 0.8" } },
						{100, new List<string> {"weather.cloud_coverage 0.22" } },
						{110, new List<string> {"weather.cloud_coverage 0.24" } },
						{120, new List<string> {"weather.cloud_coverage 0.26" } },
						{130, new List<string> {"weather.cloud_coverage 0.28" } },
						{140, new List<string> {"weather.cloud_coverage 0.3", "weather.wind 0.5" } },
						{150, new List<string> {"weather.cloud_coverage 0.32" } },
						{160, new List<string> {"weather.cloud_coverage 0.34" } },
						{170, new List<string> {"weather.cloud_coverage 0.36", "weather.cloud_size 1.25" } },
						{180, new List<string> {"weather.cloud_coverage 0.38" } },
						{190, new List<string> {"weather.cloud_coverage 0.4", "weather.wind 0.6" } },
						{300, new List<string> {"weather.cloud_coverage 0.42" } },
						{310, new List<string> {"weather.cloud_coverage 0.44" } },
						{320, new List<string> {"weather.cloud_coverage 0.46" } },
						{330, new List<string> {"weather.cloud_coverage 0.48" } },
						{340, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_attenuation 0.1" } },
						{350, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_attenuation -1" } },
						{360, new List<string> {"weather.cloud_coverage 0.48" } },
						{370, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_size 1.5" } },
						{380, new List<string> {"weather.cloud_coverage 0.52" } },
						{390, new List<string> {"weather.cloud_coverage 0.54" } },
						{400, new List<string> {"weather.cloud_coverage 0.56" } },
						{410, new List<string> {"weather.cloud_coverage 0.58" } },
						{420, new List<string> {"weather.cloud_coverage 0.6" } },
						{430, new List<string> {"weather.cloud_coverage 0.62" } },
						{440, new List<string> {"weather.cloud_coverage 0.64" } },
						{450, new List<string> {"weather.cloud_coverage 0.66" } },
						{460, new List<string> {"weather.cloud_coverage 0.68" } },
						{470, new List<string> {"weather.cloud_coverage 0.7" } },
						{480, new List<string> {"weather.cloud_coverage 0.72" } },
						{490, new List<string> {"weather.cloud_coverage 0.74" } },
						{500, new List<string> { "weather.cloud_coverage 0.75", "weather.cloud_attenuation 0.2" } },
						{510, new List<string> { "weather.fog 0.1" } },
						{520, new List<string> { "weather.cloud_coverage 0.76" } },
						{530, new List<string> { "weather.fog 0.04" } },
						{540, new List<string> { "weather.cloud_coverage 0.78" } },
						{550, new List<string> { "weather.fog 0.06" } },
						{560, new List<string> { "weather.cloud_coverage 0.8" } },
						{570, new List<string> { "weather.fog 0.08" } },
						{580, new List<string> { "weather.cloud_coverage 0.82" } },
						{590, new List<string> { "weather.fog 0.1" } },
                        {600, new List<string> { "weather.rain 0.01" } },
						{610, new List<string> { "weather.cloud_coverage 0.84" } },
						{620, new List<string> { "weather.fog 0.12" } },
                        {630, new List<string> { "weather.rain 0.02" } },
						{640, new List<string> { "weather.cloud_coverage 0.86" } },
						{650, new List<string> { "weather.cloud_coverage 0.88" } },
                        {660, new List<string> { "weather.rain 0.03" } },
						{670, new List<string> { "weather.cloud_coverage 0.9" } },
                        {680, new List<string> { "weather.rain 0.04" } },
						{690, new List<string> { "weather.cloud_coverage 0.92" } },
                        {700, new List<string> { "weather.rain 0.15", "weather.wind 0.4" } },
						{710, new List<string> { "weather.cloud_coverage 0.94" } },
						{720, new List<string> { "weather.cloud_coverage 0.96" } },
						{730, new List<string> { "weather.cloud_coverage 0.98" } },
						{740, new List<string> { "weather.cloud_coverage 1", "weather.cloud_attenuation 0.3" } },
						{1040, new List<string> { "weather.cloud_coverage 0.98" } },
						{1050, new List<string> { "weather.cloud_coverage 0.96" } },
						{1060, new List<string> { "weather.cloud_coverage 0.94" } },
						{1070, new List<string> { "weather.cloud_coverage 0.92", "weather.cloud_attenuation 0.2" } },
						{1080, new List<string> { "weather.cloud_coverage 0.9" } },
                        {1090, new List<string> { "weather.rain 0.24" } },
						{1100, new List<string> { "weather.cloud_coverage 0.88" } },
						{1110, new List<string> { "weather.cloud_coverage 0.86" } },
						{1120, new List<string> { "weather.cloud_coverage 0.84" } },
						{1130, new List<string> { "weather.cloud_coverage 0.82" } },
						{1140, new List<string> { "weather.cloud_coverage 0.8" } },
                        {1150, new List<string> { "weather.rain 0.23", "weather.wind 0.2" } },
						{1160, new List<string> { "weather.cloud_coverage 0.78" } },
						{1170, new List<string> { "weather.cloud_coverage 0.76" } },
						{1180, new List<string> { "weather.cloud_coverage 0.74", "weather.cloud_attenuation -1" } },
						{1190, new List<string> { "weather.cloud_coverage 0.72" } },
						{1200, new List<string> { "weather.cloud_coverage 0.7" } },
                        {1210, new List<string> { "weather.rain 0.12" } },
						{1220, new List<string> { "weather.cloud_coverage 0.68" } },
						{1230, new List<string> { "weather.cloud_coverage 0.66" } },
						{1240, new List<string> { "weather.cloud_coverage 0.64" } },
						{1250, new List<string> { "weather.cloud_coverage 0.62" } },
                        {1260, new List<string> { "weather.rain 0.07" } },
						{1270, new List<string> { "weather.cloud_coverage 0.6" } },
						{1280, new List<string> { "weather.cloud_coverage 0.58" } },
						{1290, new List<string> { "weather.cloud_coverage 0.56" } },
						{1300, new List<string> { "weather.cloud_coverage 0.54" } },
						{1310, new List<string> { "weather.cloud_coverage 0.52" } },
						{1320, new List<string> { "weather.cloud_coverage 0.5" } },
                        {1330, new List<string> { "weather.rain 0" } },
						{1350, new List<string> { "weather.cloud_coverage 0.48" } },
						{1750, new List<string> {"weather.cloud_coverage 0.48" } },
						{1760, new List<string> {"weather.cloud_coverage 0.46" } },
						{1770, new List<string> {"weather.cloud_coverage 0.44" } },
						{1780, new List<string> {"weather.cloud_coverage 0.42" } },
						{1790, new List<string> {"weather.cloud_coverage 0.4" } },
						{1900, new List<string> {"weather.cloud_coverage 0.38" } },
						{1910, new List<string> {"weather.cloud_coverage 0.36" } },
						{1920, new List<string> {"weather.cloud_coverage 0.34" } },
						{1930, new List<string> {"weather.cloud_coverage 0.32" } },
						{1940, new List<string> {"weather.cloud_coverage 0.3", "weather.wind 0.1" } },
						{1950, new List<string> {"weather.cloud_coverage 0.28" } },
						{1960, new List<string> {"weather.cloud_coverage 0.26" } },
						{2320, new List<string> {"weather.cloud_coverage 0.28" } },
						{2330, new List<string> {"weather.cloud_coverage 0.3" } },
						{2340, new List<string> {"weather.cloud_coverage 0.32" } },
						{2350, new List<string> {"weather.cloud_coverage 0.34" } },
						{2360, new List<string> {"weather.cloud_coverage 0.36" } },
						{2370, new List<string> {"weather.cloud_coverage 0.38", "weather.wind -1" } },
						{2380, new List<string> {"weather.cloud_coverage 0.4", "weather.atmosphere_rayleigh 1.25" } },
						{2390, new List<string> {"weather.cloud_coverage 0.42" } },
						{2400, new List<string> {"weather.cloud_coverage 0.44" } },
						{2410, new List<string> {"weather.cloud_coverage 0.46" } },
						{2420, new List<string> {"weather.cloud_coverage 0.48" } },
						{2430, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_attenuation 0.1" } },
						{2440, new List<string> {"weather.cloud_coverage 0.52" } },
						{2450, new List<string> {"weather.cloud_coverage 0.54" } },
						{2460, new List<string> {"weather.cloud_coverage 0.56" } },
						{2470, new List<string> {"weather.cloud_coverage 0.58" } },
						{2480, new List<string> {"weather.cloud_coverage 0.6" } },
						{2650, new List<string> {"weather.cloud_coverage 0.58" } },
						{2660, new List<string> {"weather.cloud_coverage 0.56" } },
						{2670, new List<string> {"weather.cloud_coverage 0.54" } },
						{2680, new List<string> {"weather.cloud_coverage 0.52" } },
						{2690, new List<string> {"weather.cloud_coverage 0.5", "weather.cloud_attenuation -1" } },
						{2700, new List<string> {"weather.cloud_coverage 0.48" } },
						{2710, new List<string> {"weather.cloud_coverage 0.46" } },
						{2720, new List<string> {"weather.cloud_coverage 0.44" } },
						{2730, new List<string> {"weather.cloud_coverage 0.42" } },
						{2740, new List<string> {"weather.cloud_coverage 0.4", "weather.atmosphere_rayleigh -1" } },
						{2750, new List<string> {"weather.cloud_coverage 0.38" } },
						{2760, new List<string> {"weather.cloud_coverage 0.36" } },
						{2770, new List<string> {"weather.cloud_coverage 0.34" } },
						{2930, new List<string> {"weather.cloud_coverage 0.32" } },
						{2940, new List<string> {"weather.cloud_coverage 0.3" } },
						{2950, new List<string> {"weather.cloud_coverage 0.28" } },
						{2960, new List<string> {"weather.cloud_coverage 0.26" } },
						{2970, new List<string> {"weather.cloud_coverage 0.24" } },
						{2980, new List<string> {"weather.cloud_coverage 0.22" } },
						{2990, new List<string> {"weather.cloud_coverage 0.2", "weather.cloud_opacity 0.9", "weather.cloud_size 0.9" } },
						{3100, new List<string> {"weather.cloud_coverage 0.18", "weather.cloud_size 1.25" } },
						{3110, new List<string> {"weather.cloud_coverage 0.16" } },
						{3120, new List<string> {"weather.cloud_coverage 0.14" } },
						{3130, new List<string> {"weather.cloud_coverage 0.12" } },
						{3140, new List<string> {"weather.cloud_coverage 0.1", "weather.cloud_opacity -1", "weather.cloud_size -1" } },
						{3150, new List<string> {"weather.cloud_coverage 0.08" } },
						{3160, new List<string> {"weather.cloud_coverage 0.06" } },
						{3170, new List<string> {"weather.cloud_coverage 0.04" } },
						{3180, new List<string> {"weather.cloud_coverage 0.02" } },								
                        {3190, new List<string> {"weather.cloud_coverage 0", "weather.reset", "STOP" } },
                    }
                    },
					{"clear", new Dictionary<int, List<string>>
                    {
                        {5, new List<string> {"weather.cloud_opacity 0.99" } },
                        {10, new List<string> {"weather.cloud_opacity 0.98" } },
                        {15, new List<string> {"weather.cloud_opacity 0.97" } },
                        {20, new List<string> {"weather.cloud_opacity 0.96" } },
                        {25, new List<string> {"weather.cloud_opacity 0.95" } },
                        {30, new List<string> {"weather.cloud_opacity 0.94" } },
                        {35, new List<string> {"weather.cloud_opacity 0.93" } },
                        {40, new List<string> {"weather.cloud_opacity 0.92" } },
                        {45, new List<string> {"weather.cloud_opacity 0.91" } },
                        {50, new List<string> {"weather.cloud_opacity 0.9" } },
                        {55, new List<string> {"weather.cloud_opacity 0.89" } },
                        {60, new List<string> {"weather.cloud_opacity 0.88" } },
                        {65, new List<string> {"weather.cloud_opacity 0.87" } },
                        {70, new List<string> {"weather.cloud_opacity 0.86" } },
                        {75, new List<string> {"weather.cloud_opacity 0.85" } },
                        {80, new List<string> {"weather.cloud_opacity 0.84" } },
                        {85, new List<string> {"weather.cloud_opacity 0.83" } },
                        {90, new List<string> {"weather.cloud_opacity 0.82" } },
                        {95, new List<string> {"weather.cloud_opacity 0.81" } },
                        {200, new List<string> {"weather.cloud_opacity 0.8", "weather.atmosphere_contrast 1.5" } },
                        {205, new List<string> {"weather.cloud_opacity 0.79" } },
                        {210, new List<string> {"weather.cloud_opacity 0.78" } },
                        {215, new List<string> {"weather.cloud_opacity 0.77" } },
                        {220, new List<string> {"weather.cloud_opacity 0.76" } },
                        {225, new List<string> {"weather.cloud_opacity 0.75" } },
                        {230, new List<string> {"weather.cloud_opacity 0.74" } },
                        {235, new List<string> {"weather.cloud_opacity 0.73" } },
                        {240, new List<string> {"weather.cloud_opacity 0.72" } },
                        {245, new List<string> {"weather.cloud_opacity 0.71" } },
                        {250, new List<string> {"weather.cloud_opacity 0.70" } },
                        {255, new List<string> {"weather.cloud_opacity 0.69" } },
                        {260, new List<string> {"weather.cloud_opacity 0.68" } },
                        {265, new List<string> {"weather.cloud_opacity 0.67" } },
                        {270, new List<string> {"weather.cloud_opacity 0.66" } },
                        {275, new List<string> {"weather.cloud_opacity 0.65" } },
                        {280, new List<string> {"weather.cloud_opacity 0.64" } },
                        {285, new List<string> {"weather.cloud_opacity 0.63" } },
                        {290, new List<string> {"weather.cloud_opacity 0.62" } },
                        {295, new List<string> {"weather.cloud_opacity 0.61" } },
                        {400, new List<string> {"weather.cloud_opacity 0.6", "weather.atmosphere_contrast 2.0" } },
                        {405, new List<string> {"weather.cloud_opacity 0.59" } },
                        {410, new List<string> {"weather.cloud_opacity 0.58" } },
                        {415, new List<string> {"weather.cloud_opacity 0.57" } },
                        {420, new List<string> {"weather.cloud_opacity 0.56" } },
                        {425, new List<string> {"weather.cloud_opacity 0.55" } },
                        {430, new List<string> {"weather.cloud_opacity 0.54" } },
                        {435, new List<string> {"weather.cloud_opacity 0.53" } },
                        {440, new List<string> {"weather.cloud_opacity 0.52" } },
                        {445, new List<string> {"weather.cloud_opacity 0.51" } },
                        {450, new List<string> {"weather.cloud_opacity 0.5" } },
                        {455, new List<string> {"weather.cloud_opacity 0.49" } },
                        {460, new List<string> {"weather.cloud_opacity 0.48" } },
                        {465, new List<string> {"weather.cloud_opacity 0.47" } },
                        {470, new List<string> {"weather.cloud_opacity 0.46" } },
                        {475, new List<string> {"weather.cloud_opacity 0.45" } },
                        {480, new List<string> {"weather.cloud_opacity 0.44" } },
                        {485, new List<string> {"weather.cloud_opacity 0.43" } },
                        {490, new List<string> {"weather.cloud_opacity 0.42" } },
                        {495, new List<string> {"weather.cloud_opacity 0.41" } },
                        {600, new List<string> {"weather.cloud_opacity 0.4" } },
                        {605, new List<string> {"weather.cloud_opacity 0.39" } },
                        {610, new List<string> {"weather.cloud_opacity 0.38" } },
                        {615, new List<string> {"weather.cloud_opacity 0.37" } },
                        {620, new List<string> {"weather.cloud_opacity 0.36" } },
                        {625, new List<string> {"weather.cloud_opacity 0.35" } },
                        {630, new List<string> {"weather.cloud_opacity 0.34" } },
                        {635, new List<string> {"weather.cloud_opacity 0.33", } },
                        {640, new List<string> {"weather.cloud_opacity 0.32" } },
                        {645, new List<string> {"weather.cloud_opacity 0.31" } },
                        {650, new List<string> {"weather.cloud_opacity 0.3" } },
                        {655, new List<string> {"weather.cloud_opacity 0.29" } },
                        {660, new List<string> {"weather.cloud_opacity 0.28" } },
                        {665, new List<string> {"weather.cloud_opacity 0.27" } },
                        {670, new List<string> {"weather.cloud_opacity 0.26" } },
                        {675, new List<string> {"weather.cloud_opacity 0.25" } },
                        {680, new List<string> {"weather.cloud_opacity 0.24" } },
                        {685, new List<string> {"weather.cloud_opacity 0.23" } },
                        {690, new List<string> {"weather.cloud_opacity 0.22" } },
                        {695, new List<string> {"weather.cloud_opacity 0.21" } },
                        {900, new List<string> {"weather.cloud_opacity 0.2" } },
                        {905, new List<string> {"weather.cloud_opacity 0.19" } },
                        {910, new List<string> {"weather.cloud_opacity 0.18" } },
                        {915, new List<string> {"weather.cloud_opacity 0.17" } },
                        {920, new List<string> {"weather.cloud_opacity 0.16" } },
                        {925, new List<string> {"weather.cloud_opacity 0.15" } },
                        {930, new List<string> {"weather.cloud_opacity 0.14" } },
                        {935, new List<string> {"weather.cloud_opacity 0.13" } },
                        {940, new List<string> {"weather.cloud_opacity 0.12" } },
                        {945, new List<string> {"weather.cloud_opacity 0.11" } },
                        {950, new List<string> {"weather.cloud_opacity 0.1" } },
                        {955, new List<string> {"weather.cloud_opacity 0.09" } },
                        {960, new List<string> {"weather.cloud_opacity 0.08" } },
                        {965, new List<string> {"weather.cloud_opacity 0.07" } },
                        {970, new List<string> {"weather.cloud_opacity 0.06" } },
                        {975, new List<string> {"weather.cloud_opacity 0.05" } },
                        {980, new List<string> {"weather.cloud_opacity 0.04" } },
                        {985, new List<string> {"weather.cloud_opacity 0.03" } },
                        {990, new List<string> {"weather.cloud_opacity 0.02" } },
                        {995, new List<string> {"weather.cloud_opacity 0.01" } },
                        {2100, new List<string> {"weather.cloud_opacity 0" } },
                        {2300, new List<string> {"weather.cloud_opacity 0.01" } },
                        {2305, new List<string> {"weather.cloud_opacity 0.02" } },
                        {2310, new List<string> {"weather.cloud_opacity 0.03" } },
                        {2315, new List<string> {"weather.cloud_opacity 0.04" } },
                        {2320, new List<string> {"weather.cloud_opacity 0.05" } },
                        {2325, new List<string> {"weather.cloud_opacity 0.06" } },
                        {2330, new List<string> {"weather.cloud_opacity 0.07" } },
                        {2335, new List<string> {"weather.cloud_opacity 0.08" } },
                        {2340, new List<string> {"weather.cloud_opacity 0.09" } },
                        {2345, new List<string> {"weather.cloud_opacity 0.1" } },
                        {2350, new List<string> {"weather.cloud_opacity 0.11" } },
                        {2355, new List<string> {"weather.cloud_opacity 0.12" } },
                        {2360, new List<string> {"weather.cloud_opacity 0.13" } },
                        {2365, new List<string> {"weather.cloud_opacity 0.14" } },
                        {2370, new List<string> {"weather.cloud_opacity 0.15" } },
                        {2375, new List<string> {"weather.cloud_opacity 0.16" } },
                        {2380, new List<string> {"weather.cloud_opacity 0.17" } },
                        {2385, new List<string> {"weather.cloud_opacity 0.18" } },
                        {2390, new List<string> {"weather.cloud_opacity 0.19" } },
                        {2395, new List<string> {"weather.cloud_opacity 0.2" } },
                        {2600, new List<string> {"weather.cloud_opacity 0.21" } },
                        {2605, new List<string> {"weather.cloud_opacity 0.22" } },
                        {2610, new List<string> {"weather.cloud_opacity 0.23" } },
                        {2615, new List<string> {"weather.cloud_opacity 0.24" } },
                        {2620, new List<string> {"weather.cloud_opacity 0.25" } },
                        {2625, new List<string> {"weather.cloud_opacity 0.26" } },
                        {2630, new List<string> {"weather.cloud_opacity 0.27" } },
                        {2635, new List<string> {"weather.cloud_opacity 0.28" } },
                        {2640, new List<string> {"weather.cloud_opacity 0.29" } },
                        {2645, new List<string> {"weather.cloud_opacity 0.3" } },
                        {2650, new List<string> {"weather.cloud_opacity 0.31" } },
                        {2655, new List<string> {"weather.cloud_opacity 0.32" } },
                        {2660, new List<string> {"weather.cloud_opacity 0.33" } },
                        {2665, new List<string> {"weather.cloud_opacity 0.34" } },
                        {2670, new List<string> {"weather.cloud_opacity 0.35" } },
                        {2675, new List<string> {"weather.cloud_opacity 0.36" } },
                        {2680, new List<string> {"weather.cloud_opacity 0.37" } },
                        {2685, new List<string> {"weather.cloud_opacity 0.38" } },
                        {2690, new List<string> {"weather.cloud_opacity 0.39" } },
                        {2695, new List<string> {"weather.cloud_opacity 0.4" } },
                        {2900, new List<string> {"weather.cloud_opacity 0.41" } },
                        {2905, new List<string> {"weather.cloud_opacity 0.42" } },
                        {2910, new List<string> {"weather.cloud_opacity 0.43" } },
                        {2915, new List<string> {"weather.cloud_opacity 0.44" } },
                        {2920, new List<string> {"weather.cloud_opacity 0.45" } },
                        {2925, new List<string> {"weather.cloud_opacity 0.46" } },
                        {2930, new List<string> {"weather.cloud_opacity 0.47" } },
                        {2935, new List<string> {"weather.cloud_opacity 0.48" } },
                        {2940, new List<string> {"weather.cloud_opacity 0.49" } },
                        {2945, new List<string> {"weather.cloud_opacity 0.5" } },
                        {2950, new List<string> {"weather.cloud_opacity 0.51" } },
                        {2955, new List<string> {"weather.cloud_opacity 0.52" } },
                        {2960, new List<string> {"weather.cloud_opacity 0.53" } },
                        {2965, new List<string> {"weather.cloud_opacity 0.54" } },
                        {2970, new List<string> {"weather.cloud_opacity 0.55" } },
                        {2975, new List<string> {"weather.cloud_opacity 0.56" } },
                        {2980, new List<string> {"weather.cloud_opacity 0.57" } },
                        {2985, new List<string> {"weather.cloud_opacity 0.58" } },
                        {2990, new List<string> {"weather.cloud_opacity 0.59" } },
                        {2995, new List<string> {"weather.cloud_opacity 0.6", "weather.atmosphere_contrast 1.5" } },
                        {3200, new List<string> {"weather.cloud_opacity 0.61" } },
                        {3205, new List<string> {"weather.cloud_opacity 0.62" } },
                        {3210, new List<string> {"weather.cloud_opacity 0.63" } },
                        {3215, new List<string> {"weather.cloud_opacity 0.64" } },
                        {3220, new List<string> {"weather.cloud_opacity 0.65" } },
                        {3225, new List<string> {"weather.cloud_opacity 0.66" } },
                        {3230, new List<string> {"weather.cloud_opacity 0.67" } },
                        {3235, new List<string> {"weather.cloud_opacity 0.68" } },
                        {3240, new List<string> {"weather.cloud_opacity 0.69" } },
                        {3245, new List<string> {"weather.cloud_opacity 0.7" } },
                        {3250, new List<string> {"weather.cloud_opacity 0.71" } },
                        {3255, new List<string> {"weather.cloud_opacity 0.72" } },
                        {3260, new List<string> {"weather.cloud_opacity 0.73" } },
                        {3265, new List<string> {"weather.cloud_opacity 0.74" } },
                        {3270, new List<string> {"weather.cloud_opacity 0.75" } },
                        {3275, new List<string> {"weather.cloud_opacity 0.76" } },
                        {3280, new List<string> {"weather.cloud_opacity 0.77" } },
                        {3285, new List<string> {"weather.cloud_opacity 0.78" } },
                        {3290, new List<string> {"weather.cloud_opacity 0.79" } },
                        {3295, new List<string> {"weather.cloud_opacity 0.80" } },
                        {3500, new List<string> {"weather.cloud_opacity 0.81" } },
                        {3505, new List<string> {"weather.cloud_opacity 0.82" } },
                        {3510, new List<string> {"weather.cloud_opacity 0.83" } },
                        {3515, new List<string> {"weather.cloud_opacity 0.84" } },
                        {3520, new List<string> {"weather.cloud_opacity 0.85" } },
                        {3525, new List<string> {"weather.cloud_opacity 0.86" } },
                        {3530, new List<string> {"weather.cloud_opacity 0.87" } },
                        {3535, new List<string> {"weather.cloud_opacity 0.88" } },
                        {3540, new List<string> {"weather.cloud_opacity 0.89" } },
                        {3545, new List<string> {"weather.cloud_opacity 0.9" } },
                        {3550, new List<string> {"weather.cloud_opacity 0.91" } },
                        {3555, new List<string> {"weather.cloud_opacity 0.92" } },
                        {3560, new List<string> {"weather.cloud_opacity 0.93" } },
                        {3565, new List<string> {"weather.cloud_opacity 0.94" } },
                        {3570, new List<string> {"weather.cloud_opacity 0.95" } },
                        {3575, new List<string> {"weather.cloud_opacity 0.96" } },
                        {3580, new List<string> {"weather.cloud_opacity 0.97" } },
                        {3585, new List<string> {"weather.cloud_opacity 0.98" } },
                        {3590, new List<string> {"weather.cloud_opacity 0.99", "weather.atmosphere_contrast -1" } },
                        {3595, new List<string> {"weather.cloud_opacity 1", "weather.reset", "STOP" } },
                    }
                    },
					{"volcanicsky", new Dictionary<int, List<string>>
                    {
                        {5, new List<string> {"weather.atmosphere_rayleigh 1.5" } },
                        {230, new List<string> {"weather.atmosphere_rayleigh 2" } },
                        {330, new List<string> {"weather.atmosphere_rayleigh 2.5" } },
                        {460, new List<string> {"weather.atmosphere_rayleigh 3" } },
                        {560, new List<string> {"weather.atmosphere_rayleigh 3.5" } },
                        {690, new List<string> {"weather.atmosphere_rayleigh 4" } },
                        {790, new List<string> {"weather.atmosphere_rayleigh 4.5" } },
                        {1020, new List<string> {"weather.atmosphere_rayleigh 5" } },
                        {1120, new List<string> {"weather.atmosphere_rayleigh 5.5" } },
                        {1250, new List<string> {"weather.atmosphere_rayleigh 6" } },
                        {1350, new List<string> {"weather.atmosphere_rayleigh 6.5" } },
                        {1450, new List<string> {"weather.atmosphere_rayleigh 7" } },
                        {1490, new List<string> {"weather.atmosphere_rayleigh 7.5" } },
                        {1510, new List<string> {"weather.atmosphere_rayleigh 8" } },
                        {1520, new List<string> {"weather.atmosphere_rayleigh 8.5" } },
                        {1540, new List<string> {"weather.atmosphere_rayleigh 9" } },
                        {1640, new List<string> {"weather.atmosphere_rayleigh 9.5" } },
                        {1770, new List<string> {"weather.atmosphere_rayleigh 10" } },
                        {2100, new List<string> {"weather.atmosphere_rayleigh 9.5" } },
                        {2200, new List<string> {"weather.atmosphere_rayleigh 9" } },
                        {2270, new List<string> {"weather.atmosphere_rayleigh 8.5" } },
                        {2330, new List<string> {"weather.atmosphere_rayleigh 8" } },
                        {2380, new List<string> {"weather.atmosphere_rayleigh 7.5" } },
                        {2460, new List<string> {"weather.atmosphere_rayleigh 7" } },
                        {2530, new List<string> {"weather.atmosphere_rayleigh 6.5" } },
                        {2590, new List<string> {"weather.atmosphere_rayleigh 6" } },
                        {2990, new List<string> {"weather.atmosphere_rayleigh 5.5" } },
                        {3020, new List<string> {"weather.atmosphere_rayleigh 5" } },
                        {3080, new List<string> {"weather.atmosphere_rayleigh 4.5" } },
                        {3150, new List<string> {"weather.atmosphere_rayleigh 4" } },
                        {3190, new List<string> {"weather.atmosphere_rayleigh 3.5" } },
                        {3280, new List<string> {"weather.atmosphere_rayleigh 3" } },
                        {3290, new List<string> {"weather.atmosphere_rayleigh 2.5" } },
                        {3310, new List<string> {"weather.atmosphere_rayleigh 2" } },
                        {3440, new List<string> {"weather.atmosphere_rayleigh 1.5" } },
                        {3570, new List<string> {"weather.atmosphere_rayleigh -1" } },
                        {3600, new List<string> {"weather.cloud_opacity 1", "weather.reset", "STOP" } },
                    }
                    },
					{"smokeysky", new Dictionary<int, List<string>>
                    {
                        {5, new List<string> {"weather.atmosphere_rayleigh 1" } },
						{10, new List<string> {"weather.cloud_coverage 0.1", "weather.cloud_opacity 0.75", "weather.cloud_size 1.5" } },
                        {230, new List<string> {"weather.atmosphere_rayleigh 2", "sv weather.atmosphere_mie 0.5", "atmosphere_directionality 0.5" } },
						{235, new List<string> {"weather.cloud_coverage 0.2", "weather.cloud_opacity 0.5", "weather.cloud_size 2.0" } },
                        {460, new List<string> {"weather.atmosphere_rayleigh 3", "sv weather.atmosphere_mie 0.25", "atmosphere_directionality 0.25" } },
                        {690, new List<string> {"weather.atmosphere_rayleigh 4" } },
                        {1020, new List<string> {"weather.atmosphere_rayleigh 5", "sv weather.atmosphere_mie 0.1", "atmosphere_directionality 0.1" } },
                        {1150, new List<string> {"weather.atmosphere_rayleigh 5.5" } },
                        {3020, new List<string> {"weather.atmosphere_rayleigh 5" } },
                        {3150, new List<string> {"weather.atmosphere_rayleigh 4" } },
                        {3280, new List<string> {"weather.atmosphere_rayleigh 3", "sv weather.atmosphere_mie 0.25", "atmosphere_directionality 0.25" } },
                        {3310, new List<string> {"weather.atmosphere_rayleigh 2", "sv weather.atmosphere_mie 0.5", "atmosphere_directionality 0.5" } },
						{3315, new List<string> {"weather.cloud_coverage 0.1", "weather.cloud_opacity 0.75", "weather.cloud_size 1.5" } },
                        {3440, new List<string> {"weather.atmosphere_rayleigh 1", "sv weather.atmosphere_mie -1", "atmosphere_directionality -1" } },
						{3510, new List<string> {"weather.cloud_coverage 0", "weather.cloud_opacity -1", "weather.cloud_size -1" } },
                        {3570, new List<string> {"weather.atmosphere_rayleigh -1" } },
                        {3600, new List<string> {"weather.cloud_opacity 1", "weather.reset", "STOP" } },
                    }
                    },
					{"windy", new Dictionary<int, List<string>>
                    {
                        {5, new List<string> {"weather.wind 0.01" } },
                        {10, new List<string> {"weather.wind 0.02" } },
                        {15, new List<string> {"weather.wind 0.03" } },
                        {20, new List<string> {"weather.wind 0.04" } },
                        {25, new List<string> {"weather.wind 0.05" } },
                        {30, new List<string> {"weather.wind 0.06" } },
                        {35, new List<string> {"weather.wind 0.07" } },
                        {40, new List<string> {"weather.wind 0.08" } },
                        {45, new List<string> {"weather.wind 0.09" } },
                        {50, new List<string> {"weather.wind 0.10" } },
                        {55, new List<string> {"weather.wind 0.11" } },
                        {60, new List<string> {"weather.wind 0.12" } },
                        {65, new List<string> {"weather.wind 0.13" } },
                        {70, new List<string> {"weather.wind 0.14" } },
                        {75, new List<string> {"weather.wind 0.15" } },
                        {80, new List<string> {"weather.wind 0.16" } },
                        {85, new List<string> {"weather.wind 0.17" } },
                        {90, new List<string> {"weather.wind 0.18" } },
                        {95, new List<string> {"weather.wind 0.19" } },
                        {100, new List<string> {"weather.wind 0.20" } },
                        {105, new List<string> {"weather.wind 0.21" } },
                        {110, new List<string> {"weather.wind 0.22" } },
                        {115, new List<string> {"weather.wind 0.23" } },
                        {120, new List<string> {"weather.wind 0.24" } },
                        {125, new List<string> {"weather.wind 0.25" } },
                        {130, new List<string> {"weather.wind 0.26" } },
                        {135, new List<string> {"weather.wind 0.27" } },
                        {140, new List<string> {"weather.wind 0.28" } },
                        {145, new List<string> {"weather.wind 0.29" } },
                        {150, new List<string> {"weather.wind 0.30" } },
                        {155, new List<string> {"weather.wind 0.31" } },
                        {160, new List<string> {"weather.wind 0.32" } },
                        {165, new List<string> {"weather.wind 0.33" } },
                        {170, new List<string> {"weather.wind 0.34" } },
                        {175, new List<string> {"weather.wind 0.35" } },
                        {180, new List<string> {"weather.wind 0.36" } },
                        {185, new List<string> {"weather.wind 0.39" } },
                        {190, new List<string> {"weather.wind 0.42" } },
                        {195, new List<string> {"weather.wind 0.47", "weather.fog 0.1" } },
                        {200, new List<string> {"weather.wind 0.54" } },
                        {205, new List<string> {"weather.wind 0.58" } },
                        {210, new List<string> {"weather.wind 0.59" } },
                        {215, new List<string> {"weather.wind 0.60" } },
                        {220, new List<string> {"weather.wind 0.61" } },
                        {225, new List<string> {"weather.wind 0.62" } },
                        {230, new List<string> {"weather.wind 0.63" } },
                        {235, new List<string> {"weather.wind 0.64" } },
                        {240, new List<string> {"weather.wind 0.65" } },
                        {245, new List<string> {"weather.wind 0.66" } },
                        {250, new List<string> {"weather.wind 0.67" } },
                        {255, new List<string> {"weather.wind 0.68" } },
                        {260, new List<string> {"weather.wind 0.69" } },
                        {265, new List<string> {"weather.wind 0.70" } },
                        {270, new List<string> {"weather.wind 0.71" } },
                        {275, new List<string> {"weather.wind 0.72" } },
                        {280, new List<string> {"weather.wind 0.73" } },
                        {285, new List<string> {"weather.wind 0.74" } },
                        {290, new List<string> {"weather.wind 0.75" } },
                        {295, new List<string> {"weather.wind 0.76" } },
                        {300, new List<string> {"weather.wind 0.77" } },
                        {305, new List<string> {"weather.wind 0.78" } },
                        {310, new List<string> {"weather.wind 0.79" } },
                        {315, new List<string> {"weather.wind 0.80" } },
                        {320, new List<string> {"weather.wind 0.82" } },
                        {325, new List<string> {"weather.wind 0.84" } },
                        {330, new List<string> {"weather.wind 0.86" } },
                        {335, new List<string> {"weather.wind 0.88", "weather.fog 0.1" } },
                        {340, new List<string> {"weather.wind 0.90" } },
                        {345, new List<string> {"weather.wind 0.92" } },
                        {350, new List<string> {"weather.wind 0.93" } },
                        {355, new List<string> {"weather.wind 0.94" } },
                        {360, new List<string> {"weather.wind 0.95" } },
                        {365, new List<string> {"weather.wind 0.96" } },
                        {370, new List<string> {"weather.wind 0.97" } },
                        {375, new List<string> {"weather.wind 0.98" } },
                        {380, new List<string> {"weather.wind 0.99" } },
                        {385, new List<string> {"weather.wind 1" } },
                        {420, new List<string> {"weather.wind 0.99" } },
                        {425, new List<string> {"weather.wind 0.98" } },
                        {430, new List<string> {"weather.wind 0.97" } },
                        {435, new List<string> {"weather.wind 0.96" } },
                        {440, new List<string> {"weather.wind 0.95" } },
                        {445, new List<string> {"weather.wind 0.94" } },
                        {450, new List<string> {"weather.wind 0.93" } },
                        {455, new List<string> {"weather.wind 0.92" } },
                        {460, new List<string> {"weather.wind 0.91" } },
                        {470, new List<string> {"weather.wind 0.9" } },
                        {480, new List<string> {"weather.wind 0.89" } },
                        {490, new List<string> {"weather.wind 0.88" } },
                        {2000, new List<string> {"weather.wind 0.87" } },
                        {2005, new List<string> {"weather.wind 0.86" } },
                        {2010, new List<string> {"weather.wind 0.85" } },
                        {2015, new List<string> {"weather.wind 0.84" } },
                        {2020, new List<string> {"weather.wind 0.83" } },
                        {2025, new List<string> {"weather.wind 0.82" } },
                        {2030, new List<string> {"weather.wind 0.81" } },
                        {2035, new List<string> {"weather.wind 0.80" } },
                        {2040, new List<string> {"weather.wind 0.79" } },
                        {2045, new List<string> {"weather.wind 0.78" } },
                        {2050, new List<string> {"weather.wind 0.77" } },
                        {2055, new List<string> {"weather.wind 0.76" } },
                        {2060, new List<string> {"weather.wind 0.75" } },
                        {2065, new List<string> {"weather.wind 0.74" } },
                        {2070, new List<string> {"weather.wind 0.73" } },
                        {2075, new List<string> {"weather.wind 0.72" } },
                        {2080, new List<string> {"weather.wind 0.71" } },
                        {2085, new List<string> {"weather.wind 0.7" } },
                        {2090, new List<string> {"weather.wind 0.69" } },
                        {2095, new List<string> {"weather.wind 0.68" } },
                        {2100, new List<string> {"weather.wind 0.67" } },
                        {2105, new List<string> {"weather.wind 0.66" } },
                        {2110, new List<string> {"weather.wind 0.65" } },
                        {2115, new List<string> {"weather.wind 0.64" } },
                        {2120, new List<string> {"weather.wind 0.63" } },
                        {2125, new List<string> {"weather.wind 0.62" } },
                        {2130, new List<string> {"weather.wind 0.61" } },
                        {2135, new List<string> {"weather.wind 0.6" } },
                        {2140, new List<string> {"weather.wind 0.59" } },
                        {2145, new List<string> {"weather.wind 0.58" } },
                        {2150, new List<string> {"weather.wind 0.57" } },
                        {2155, new List<string> {"weather.wind 0.56" } },
                        {2160, new List<string> {"weather.wind 0.55" } },
                        {2165, new List<string> {"weather.wind 0.54" } },
                        {2170, new List<string> {"weather.wind 0.53" } },
                        {2175, new List<string> {"weather.wind 0.52" } },
                        {2180, new List<string> {"weather.wind 0.51" } },
                        {2185, new List<string> {"weather.wind 0.5", "weather.fog 0.1" } },
                        {2190, new List<string> {"weather.wind 0.49" } },
                        {2195, new List<string> {"weather.wind 0.48" } },
                        {2200, new List<string> {"weather.wind 0.47" } },
                        {2205, new List<string> {"weather.wind 0.46" } },
                        {2210, new List<string> {"weather.wind 0.45" } },
                        {2215, new List<string> {"weather.wind 0.44" } },
                        {2220, new List<string> {"weather.wind 0.43" } },
                        {2225, new List<string> {"weather.wind 0.42" } },
                        {2230, new List<string> {"weather.wind 0.41" } },
                        {2235, new List<string> {"weather.wind 0.4" } },
                        {2240, new List<string> {"weather.wind 0.39" } },
                        {2245, new List<string> {"weather.wind 0.38" } },
                        {2250, new List<string> {"weather.wind 0.37" } },
                        {2280, new List<string> {"weather.wind 0.36" } },
                        {2310, new List<string> {"weather.wind 0.35" } },
                        {2320, new List<string> {"weather.wind 0.34" } },
                        {2330, new List<string> {"weather.wind 0.33" } },
                        {2335, new List<string> {"weather.wind 0.32" } },
                        {2340, new List<string> {"weather.wind 0.31" } },
                        {2345, new List<string> {"weather.wind 0.3" } },
                        {2350, new List<string> {"weather.wind 0.29" } },
                        {2355, new List<string> {"weather.wind 0.28" } },
                        {2360, new List<string> {"weather.wind 0.27" } },
                        {2365, new List<string> {"weather.wind 0.26" } },
                        {2370, new List<string> {"weather.wind 0.25" } },
                        {2375, new List<string> {"weather.wind 0.24" } },
                        {2380, new List<string> {"weather.wind 0.23" } },
                        {2385, new List<string> {"weather.wind 0.22" } },
                        {2390, new List<string> {"weather.wind 0.21" } },
                        {2395, new List<string> {"weather.wind 0.2" } },
                        {2400, new List<string> {"weather.wind 0.19" } },
                        {2405, new List<string> {"weather.wind 0.18" } },
                        {2410, new List<string> {"weather.wind 0.17" } },
                        {2415, new List<string> {"weather.wind 0.16" } },
                        {2420, new List<string> {"weather.wind 0.15" } },
                        {2425, new List<string> {"weather.wind 0.14" } },
                        {2430, new List<string> {"weather.wind 0.13" } },
                        {2435, new List<string> {"weather.wind 0.12" } },
                        {2440, new List<string> {"weather.wind 0.11" } },
                        {2445, new List<string> {"weather.wind 0.1" } },
                        {2450, new List<string> {"weather.wind 0.09" } },
                        {2455, new List<string> {"weather.wind 0.08" } },
                        {2460, new List<string> {"weather.wind 0.07" } },
                        {2465, new List<string> {"weather.wind 0.06" } },
                        {2470, new List<string> {"weather.wind 0.05" } },
                        {2475, new List<string> {"weather.wind 0.04" } },
                        {2480, new List<string> {"weather.wind 0.03" } },
                        {2485, new List<string> {"weather.wind 0.02" } },
                        {2490, new List<string> {"weather.wind 0.01", "weather.fog 0" } },
                        {2495, new List<string> {"weather.wind 0", "weather.reset", "STOP" } },
                    }
                    }
            };
    }
}